<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco;

use Exception;
use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Geral\Entities\Banco;
use Geral\Entities\Empresa;
use Geral\Entities\Fornecedor;
use Geral\Entities\Municipio;
use Geral\Entities\Filial;
use Geral\Entities\Contcorr;
use Geral\Entities\Tipobaixapag;
use Teknisa\Libs\Util\DateUtil;
use Fiscal\Entities\Contapagar;
use Financeiro\Util\Util;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\GeraCnabService;

/**
 * Class GeraCnabSicrediService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco
 */
class GeraCnabSicrediService extends GeraCnabService{

    CONST NUM_LOTE_HEADER_ARQUIVO = '0000';
    CONST ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    CONST LAYOUT_ARQUIVO = '082';
    CONST ID_TP_ARQUIVO_HEADER_LOTE = '1';
    CONST ID_TP_ARQUIVO_SEGMENTO = '3';
    CONST ID_TP_ARQUIVO_TRAILER_LOTE = '5';
    CONST TIPO_INS_JUR_CPF = 1;
    CONST TIPO_INS_JUR_CNPJ = 2;
    CONST NOME_BANCO = 'SICREDI';
    CONST CDBANCO = '748';
    CONST REMESSA = 1;
    CONST RETORNO = 2;
    CONST VERSAO_LAYOUT = '060';
    CONST TIPO_OPERACAO_CREDITO = 'C';
    CONST TIPO_SERVICO = '20';
    CONST VERSAO_LOTE = '040';
    CONST VERSAO_LOTE_TITULO_BANC = '030';
    CONST SEGMENTO_A = 'A';
    CONST EXCLUSAO = 'E';
    CONST INCLUSAO = 'I';
    CONST BLOQUEIO = 'B';
    CONST MOEDA = 'BRL';
    CONST SEGMETO_B = 'B';
    CONST LOTE_SERVICO_TRAILER_ARQUIVO = '9999';
    CONST ID_TP_ARQUIVO_TRAILER = '9';
    CONST SEGMENTO_J = 'J';
    const TIPO_BAIXA_PAG_IPVA = 'V';
    const TIPO_BAIXA_PAG_GARE = 'X';
    const TIPO_BAIXA_PAG_DARJ = 'J';
    const TIPO_BAIXA_PAG_FGTS = 'U';

    private $num_header_lote = 0;
    private $seqRegLote = 0;
    private $qtdRegiLote = 0;  //Qtde de registro do lote
    private $qtdRegiArq = 0; //Quantidade de registro do arquivo
    private $qtLote = 0; //Quantidade de Lotes no arquivo
    private $vrPagtLote = 0.00; // Somatório dos valores de inclusao do lote
    private $formapgto;

    /**
     * @param Cnabpag $remessa
     * @throws Exception
     */
    public function __construct(Cnabpag $remessa){
        try {
            $this->nrorg = $remessa->getNrorg();

            $this->remessa = $remessa;

            // Carrega os atributos Filial e Empresa
            $this->filial = Filial::findOneByCdfilialAndNrorg($remessa->getCdfilial(), $remessa->getNrorg());
            $this->empresa = Empresa::findOneByCdempresaAndNrorg($remessa->getCdempresa(), $remessa->getNrorg());

            // Busca dados referente a filial se foi informada
            if ($remessa->getCdfilial()) {
                if ($this->filial->getIdtpijurfili() == Filial::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = $this->filial->getNrinsjurfili();
                $this->nomeempresa = $this->filial->getNmfilial();
            } // Se nao busca dados da Empresa
            else {
                if ($this->empresa->getIdtpijurempr() == Empresa::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = $this->empresa->getNrinsjurempr();
                $this->nomeempresa = $this->empresa->getNmempresa();
            }
            /** @var contcorr Contcorr */
            $this->contcorr = Contcorr::findOneByCdreducontAndNrorg($remessa->getCdreducont(), $remessa->getNrorg());
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $caminhoArquivo
     * @param $boletos
     * @param bool $autorizaAgendamento - Parámetro utilizando quando o banco for Itaú
     * @throws \Exception
     * @internal param $remessa
     * @return mixed
     */
    public function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false, $idopecnabpag = null){
        try{
            $file = fopen($caminhoArquivo.'.REM', "a+");
            $headerarquivo = self::geraHeaderArquivo();
            fprintf($file, "%s",$headerarquivo);

            // Agrupar boletos por tipo de baixa e por banco favorecido
            $boletosAgrup = self::agrupaBoletos($boletos, $file);
            
            //Percorre as formas de pagamento
            foreach($boletosAgrup as $segmento => $boletosFormaPgo)
            {
                
                //Percorre os bancos
                foreach($boletosFormaPgo as $key => $boletosBco)
                {
                    $headerlote  = self::geraHeaderLote($key, $boletosBco);
                    fprintf($file, "%s",$headerlote);
                    $somaboletos = 0;
                    foreach($boletosBco as $boleto)
                    {
                        if($segmento == 'SEGMENTOA'){
                            $segmentoA = self::geraSegmentoA($boleto, $idopecnabpag);
                            $segmentoB = self::geraSegmentoB($boleto, $idopecnabpag);
                            fprintf($file, "%s",$segmentoA);
                            fprintf($file, "%s",$segmentoB);
                        }
                        if($segmento == 'SEGMENTOJ'){
                            $segmentoJ = self::geraSegmentoJ($boleto, $idopecnabpag);
                            fprintf($file, "%s",$segmentoJ);
                            $segmentoJ52 = self::geraSegmentoJ52($boleto, $idopecnabpag);
                            fprintf($file, "%s",$segmentoJ52);
                        }
                        if($segmento == 'SEGMENTOO'){
                            $segmentoO = self::geraSegmentoO($boleto, $idopecnabpag);
                            fprintf($file, "%s",$segmentoO);
                        }
                        if($segmento == 'SEGMENTON'){
                            $segmentoN = self::geraSegmentoN($boleto, $idopecnabpag);
                            fprintf($file, "%s",$segmentoN);
                        }
                        
                        $somaboletos += $boleto->getVrfinalbpag();
                    }
                    $trailerLote = self::geraTraillerLote($somaboletos);
                    fprintf($file, "%s",$trailerLote);
                }
            }
            $trailerArquivo = self::geraTrailerArquivo();
            fprintf($file, "%s",$trailerArquivo);
            fclose($file);
        }
        catch (\Exception $e){
            fclose($file);
            throw $e;
        }
    }

    /** Retorna array de Boletos Agrupados por Forma de Pagamento e por código do banco favorecido*/
    private function agrupaBoletos($boletos){
        $boletosAgroup = array();

        $segmentoJ = array(Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

        $segmentoO = array(Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS);
        
        $segmentoAPIX = array('F');
        
        $segmentoJPIX = array('Z');
        
        $segmentoN = array(
            Tipobaixapag::TIPO_BAIXA_PAG_DARF_SIMPLES,
            Tipobaixapag::TIPO_BAIXA_GPS,
            Tipobaixapag::TIPO_BAIXA_PAG_DARF_NORMAL,
            self::TIPO_BAIXA_PAG_IPVA,
            self::TIPO_BAIXA_PAG_GARE,
            self::TIPO_BAIXA_PAG_DARJ,
            self::TIPO_BAIXA_PAG_FGTS
        );
        
        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto)
        {
            
            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag($boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']);
            
            $idbaixautilp = $boleto->getTpbaixapag()->getIdbaixautilp();
            
            if(in_array($idbaixautilp,$segmentoJ))
            {
                $cdbanco = self::getBanco($boleto,Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

                if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_SICREDI))
                {
                    $boletosAgroup['SEGMENTOJ'] [GeraCnabService::CODIGO_COMPENSACAO_SICREDI] [] = $boleto;
                }
                else
                {
                    $boletosAgroup['SEGMENTOJ'] ['OUTROS_BANCOS'] [] = $boleto;
                }

            }
            else
            {
                if(in_array($idbaixautilp,$segmentoO))
                {
                    $cdbanco = self::getBanco($boleto,Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS);

                    if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_SICREDI))
                    {
                        $boletosAgroup['SEGMENTOO'] [GeraCnabService::CODIGO_COMPENSACAO_SICREDI] [] = $boleto;
                    }
                    else
                    {
                        $boletosAgroup['SEGMENTOO'] ['OUTROS_BANCOS'] [] = $boleto;
                    }

                }
                elseif(in_array($idbaixautilp,$segmentoN)){
                    $cdbanco = self::getBanco($boleto, $idbaixautilp);
                    
                    if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_SICREDI)){
                        $boletosAgroup['SEGMENTON'] [GeraCnabService::CODIGO_COMPENSACAO_SICREDI] [] = $boleto;
                    }
                    else{
                        $boletosAgroup['SEGMENTON'] ['OUTROS_BANCOS'] [] = $boleto;
                    }
                    
                }
                else
                {
                    if(in_array($idbaixautilp,$segmentoAPIX))
                    {
                        $cdbanco = self::getBanco($boleto, 'F');
    
                        if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_SICREDI))
                        {
                            $boletosAgroup['SEGMENTOAPIX'] [GeraCnabService::CODIGO_COMPENSACAO_SICREDI] [] = $boleto;
                        }
                        else
                        {
                            $boletosAgroup['SEGMENTOAPIX'] ['OUTROS_BANCOS'] [] = $boleto;
                        }
    
                    }
                    else
                    {
                        if(in_array($idbaixautilp,$segmentoJPIX))
                        {
                            $cdbanco = self::getBanco($boleto, 'Z');
        
                            if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_SICREDI))
                            {
                                $boletosAgroup['SEGMENTOJPIX'] [GeraCnabService::CODIGO_COMPENSACAO_SICREDI] [] = $boleto;
                            }
                            else
                            {
                                $boletosAgroup['SEGMENTOJPIX'] ['OUTROS_BANCOS'] [] = $boleto;
                            }
        
                        }
                        else
                        {
                            $cdbanco = self::getBanco($boleto);
        
                            if($cdbanco == (GeraCnabService::CODIGO_COMPENSACAO_SICREDI))
                            {
                                $boletosAgroup['SEGMENTOA'] [GeraCnabService::CODIGO_COMPENSACAO_SICREDI] [] = $boleto;
                            }
                            else
                            {
                                $boletosAgroup['SEGMENTOA'] ['OUTROS_BANCOS'] [] = $boleto;
                            }
                        }
                    }
                }
            }
        }
        
        return $boletosAgroup;
    }

    // Registro Header de Arquivo
    /**
     * @throws Exception
     * @return string
     */
    public function geraHeaderArquivo()
    {

        try {

            $this->qtdRegiArq = 0;
            $this->qtdRegiArq++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;
            
            //Número sequêncial do Lote - Posição: 4 a 7 - Picture: 9(4)
            $linha.= self::NUM_LOTE_HEADER_ARQUIVO;
            
            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_ARQUIVO;
            
            //Brancos - Posição: 9 a 17 - Picture: X(9)
            $linha.= self::preenche_brancos('',9);
            
            //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
            $linha.= $this->tipoinscricao;
            
            //CNPJ empresa debitada - Posição: 19 a 32 - Picture: 9(14)
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);
            
            //Filler - Posição: 33 a 52 - Picture: X(9)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),20);
            }else{
                $linha.= str_repeat(' ', 20);
            }
            
            //Número da agência centralizadora - Posição: 53 a 57 - Picture: 9(05)
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),-5),5,'0',STR_PAD_LEFT);
            
            //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
            $linha.= substr($this->contcorr->getCdagencia(), -1, 1);
            
            //Número da conta - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);
            
            //Digito CC - Posição: 71 a 71 - Picture: X(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);
            
            //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: X(1)
            $linha.= str_repeat(' ', 1);

            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(strtoupper($this->nomeempresa)),0,30),30);
            
            //Nome do Banco - Posição: 103 a 132 - Picture: X(30)
            $banco = Banco::findOneByCdbancoAndNrorg($this->contcorr->getCdbanco(), $this->nrorg);
            $linha.= self::preenche_brancos(Util::campoCnabValido($banco->getNmbanco()),30);
            
            //Filler - Posição: 133 a 142 - Picture: X(10)
            $linha.= self::preenche_brancos('',10);
            
            //Código remessa/Retorno (1 = Remessa, 2 = Retorno) - Posição: 143 a 143 - Picture: 9(1)
            $linha.= self::REMESSA;
            
            //Data da geração do arquivo - Posição: 144 a 151 - Picture: 9(8)
            $linha.= self::getDataGeracao();
            
            //Hora de geração do arquvio - Posição: 152 a 157 - Picture 9(6)
            $linha.= self::getHorarioAtual();
            
            //Sequência (NSA) Picture: 9(6) - Posição: 158 a 163
            $linha.=str_pad(ltrim($this->remessa->getNrcnabpag(), '0'),6 ,'0',STR_PAD_LEFT);
            
            //Leiaute do arquivo - Posição: 164 a 166 - Picture: 9(3)
            $linha.= self::LAYOUT_ARQUIVO;
            
            //Densidade de gravação do arquivo - Posição: 167 a 171 - Picture: 9(5)
            $linha.= '01600';
            
            //Filler - Posição: 172 a 240 - Picture: X(69)
            $linha.= str_repeat(' ', 69);
            
            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdbanco
     * @param $boletos
     * @throws \Exception
     * @internal param $formaLancamento
     * @return string
     */
    public function geraHeaderLote($cdbanco, $boletos = null){

        try{
            $this->vrPagtLote = 0;
            $this->qtdRegiLote = 0;
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;
            $this->qtLote++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            //Número sequêncial do Lote - Posição: 4 a 7 - Picture: 9(4)
            $linha.= str_pad(++$this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_LOTE;

            //Tipo de operação (Fixo "C" = Crédito) - Posição: 9 a 9 - Picture: X(01)
            $linha.= self::TIPO_OPERACAO_CREDITO;

            //Tipo de Serviço - Posição: 10 a 11 - Picture: 9(02)
            $formaPagto = self::formaPagamento($cdbanco, $boletos);
            if (($formaPagto == '16')or ($formaPagto == '17') or ($formaPagto == '18') or
                ($formaPagto == '19') or ($formaPagto == '21') or
                ($formaPagto == '25') or ($formaPagto == '27') or
                ($formaPagto == '35') or ($formaPagto == '91'))
                $linha.= '22';
                else if(($formaPagto == '30'))
                    $linha.= '03';
                    else
                    $linha.= '20';
           
            //Forma de lançamento - Posição: 12 a 13 - Picture: 9(02)
            $linha.= $formaPagto;

            //Número da versão no layout do lote - Posição: 14 a 16 - Picture: 9(03)
            $linha.= '042';

            //Filler - Posição: 17 a 17 - Picture: X(01)
            $linha .= ' ';

            //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
            $linha.= $this->tipoinscricao;
            
            //CNPJ empresa debitada - Posição: 19 a 32 - Picture: 9(14)
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);

            //Código do convênio no banco
            //Posição: 33 a 52
            //Picture: X(20)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),20);
            }else{
                $linha.= str_repeat(' ', 20);
            }

            //Número da agência centralizadora - Posição: 53 a 57 - Picture: 9(05)
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),-5),5,'0',STR_PAD_LEFT);
            
            //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
            $linha.= substr(trim($this->contcorr->getCdagencia()), -1);
            
            //Número da conta - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);
            
            //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);

            //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: 9(1)
            $linha.= str_repeat(' ', 1);
            
            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(strtoupper($this->nomeempresa)),0,30),30);
            
            //Campo de mensagem genérica - Posição: 103 a 142 - Picture: X(40)
            $linha.= str_repeat(' ', 40);

            //Endereço da Empresa - Posição: 143 a 172 - Picture: X(30)
            $linha.= str_pad(substr(Util::campoCnabValido($this->empresa->getDsendeempr()),0, 30), 30);

            //Campo de mensagem genérica - Posição: 173 a 177 - Picture: X(5)
            $linha.= str_repeat('0', 5);

            //Campo de mensagem genérica - Posição: 178 a 192 - Picture: X(15)
            $linha.= str_repeat(' ', 15);

            //Cidade da Empresa - Posição: 193 a 212 - Picture: X(20)
            $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($this->empresa->getSgestado(), $this->empresa->getCdpais(), $this->empresa->getCdmunicipio());
            $linha.= str_pad(substr($municipio->getNmmunicipio(), 0, 20), 20);

            //Cep da Empresa - Posição: 213 a 220 - Picture: X(8)
            $linha.= Util::campoCepTel($this->empresa->getNrcepempr());

            //Estado da Empresa - Posição: 221 a 222 - Picture: X(2)
            $linha.= $this->empresa->getSgestado();

            //Complemento de Registro - Posição: 223 a 230 - Picture: X(08)
            $linha.= str_repeat(' ', 8);

            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ', 10);
            
            return $linha."\r\n";


        }catch(\Exception $e){
            throw $e;
        }
    }

    public function formaPagamento($cdbanco, $boletos)
    {
        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto){
            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag($boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolpag());

            $idbaixautilp = $boleto->getTpbaixapag()->getIdbaixautilp();
        }
        
        switch ($idbaixautilp)
        {
            case 'B':
                if($cdbanco == self::CODIGO_COMPENSACAO_SICREDI)
                    $this->formapgto = '30';
                else
                    $this->formapgto = '31';
                break;
            case 'G': $this->formapgto = '17';
                break;
            case 'H': $this->formapgto = '18';
                break;
            case 'I': $this->formapgto = '16';
                break;
            case 'J': $this->formapgto = '21';
                break;
            case 'K': $this->formapgto = '27';
                break;
            case 'N': $this->formapgto = '19';
                break;
            case 'O': $this->formapgto = '10';
                break;
            case 'Q': $this->formapgto = '91';
                break;
            case 'R': $this->formapgto = '13';
                break;
            case 'U': $this->formapgto = '35';
                break;
            case 'V': $this->formapgto = '25';
                break;
            case 'X': $this->formapgto = '22';
                break;
            default:
                if ($boleto->getCdbancbolp() == '748')
                {
                    $this->formapgto = '01';

                }
                else if ($boleto->getVrfinalbpag() <= $boleto->getTpbaixapag()->getVrmaxdoccnab())
                    $this->formapgto = '03';
                else
                    $this->formapgto = '41';
                break;
        }

        return $this->formapgto;
    }

    /**
     * retorna a data atual no formato ddmmYYYY
     * @return string
     */
    private function getDataGeracao(){
        $dataatual = DateUtil::getDataAtual();
        return $dataatual->format('d').$dataatual->format('m').$dataatual->format('Y');
    }

    /**
     * @return string
     */
    private function getHorarioAtual(){
        $dataatual = DateUtil::getDataAtual();
        return $dataatual->format('H').$dataatual->format('i').$dataatual->format('s');
    }

    /** Retorna Conta Corrente sem a ultima posição (dígito)
     * @return string
     */
    protected function getContaCorrenteSemDigito(){
        $contacorrente = $this->contcorr->getCdcontcorr();
        return substr($contacorrente,0,-1);
    }

    /**
     * Retorna a ultima posição da Conta Corrente (Dígito)
     * @return int|string
     */
    protected function getDigitoContaCorrente(){
        $contacorrente = $this->contcorr->getCdcontcorr();
        return substr($contacorrente,-1);
    }

    /**
     * @param $valor - valor a ser alterado
     * @param $tamanho - tamanho total da string desejado
     * @return string
     */
    private function preenche_brancos($valor, $tamanho){
        return str_pad($valor, $tamanho);
    }

    /**
     * @param $nrlanbolpag
     * @param $cdfilibpag
     * @param $nrorg
     * @return Contapagar
     */
    private function getContaPagar($nrlanbolpag, $cdfilibpag, $nrorg){
        /** @var $contaPagar Contapagar */
        $contaPagar = Contapagar::findOneByNrlanbolpagAndCdfilibpagAndNrorg($nrlanbolpag, $cdfilibpag, $nrorg);
        return $contaPagar;
    }


    /**
     * @internal param $cdbanco
     * @throws Exception
     * @return string
     * @internal param $caminho
     * @internal param $nrorg
     */
    public function geraTraillerLote($somavalores = null){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do banco na compensação - Posição: 1 a 3 - Picture: 9(3)
            $linha = self::CODIGO_COMPENSACAO_SICREDI;

            //Número de sequência do lote - Posição: 4 a 7 - Picture: 9(4)
            $linha .= str_pad($this->num_header_lote, 4, '0', STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha .= self::ID_TP_ARQUIVO_TRAILER_LOTE;

            //Filler - Posição: 9 a 17 - Picture: X(9)
            $linha .= str_repeat(' ', 9);

            //Quantidade de Registro do lote - Posição: 18 a 23 - Picture: 9(6)
            $linha .= str_pad($this->qtdRegiLote, 6, '0', STR_PAD_LEFT);

            //Somatorio dos valores a serem lançados - Posição: 24 a 41 - Picture: 9(16)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($somavalores,2)
            ),18,'0',STR_PAD_LEFT);

            //Somatório da quantidade de moeda - Posição: 42 a 56 - Picture: 9(18)
            $linha .= str_repeat('0', 18);

            //Filler - Posição: 57 a 230 - Picture: X(165)
            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha .= str_repeat(' ', 171);

            //Ocorrências - Posição: 231 a 240 - Picture: X(10)
            $linha .= str_repeat(' ', 10);

            //Zera o contador de registros do lote
            $this->qtdRegiLote = 0;

            $this->seqRegLote = 0;

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }

    }

    /**
     * @return string
     * @throws \Exception
     */
    public function geraTrailerArquivo(){

        try { //Quantidade de registro do arquivo
            $this->qtdRegiArq++;

            //Código do banco na compensação - Posição: 1 a 3 - Picture: 9(3)
            $linha = self::CODIGO_COMPENSACAO_SICREDI;

            //Número de sequência do lote - Posição: 4 a 7 - Picture: 9(4)
            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha .= self::LOTE_SERVICO_TRAILER_ARQUIVO . self::ID_TP_ARQUIVO_TRAILER;
            
            //Filler - Posição: 9 a 17 - Picture: X(9)
            $linha .= str_repeat(' ', 9);

            //Quantidade de lotes do arquivo - Posição: 18 a 23 - Picture: 9(6)
            $linha .= str_pad($this->qtLote, 6, '0', STR_PAD_LEFT);

            //Qtde total de registros do arquivo - Posição: 24 a 29 - Picture: 9(6)
            $linha .= str_pad($this->qtdRegiArq, 6, '0', STR_PAD_LEFT);
            
            //Qtde. de contas concil. - Posição: 30 a 35 - Picture: 9(6)
            $linha .= str_repeat('0', 6);
            
            //Brancos - Posição: 36 a 240 - Picture: X(205)
            $linha .= str_repeat(' ', 205);
            
            return $linha . "\r\n";
            
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function geraSegmentoJ($boleto, $idopecnabpag)
    {
        try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SICREDI;
            
            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);
            
            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;
            
            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);
            
            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= 'J';
            
            //Tipo de movimento - Posição: 15 a 17 - Picture: 9(3)
            if ($idopecnabpag == 'E')
                $linha.= '999';
            else
                $linha.= '000';

            //Código de Barras - Posição: 18 a 61 - Picture: X(44)
            if($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
                if($boleto->getCdbarrapag()){
                    $cdbanco = str_pad(substr($boleto->getCdbarrapag(),0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($boleto->getCdbarrapag(),3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($boleto->getCdbarrapag(),4,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($boleto->getCdbarrapag(),5,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($boleto->getCdbarrapag(),19,25),25,'0',STR_PAD_LEFT);
                } else {
                    $linhaBarra = '';
                    $dslinhbarpag = $boleto->getDslinhbarpag();
                    for($i = 0;$i<strlen($dslinhbarpag);$i++){
                        if(in_array($dslinhbarpag[$i],array('0','1','2','3','4','5','6','7','8','9'))){
                            $linhaBarra.= $dslinhbarpag[$i];
                        }
                    }
                    $cdbanco = str_pad(substr($linhaBarra,0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($linhaBarra,3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($linhaBarra,32,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($linhaBarra,33,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($linhaBarra,4,25),25,'0',STR_PAD_LEFT);
                }
            }
            //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
            $linha.= $cdbanco;

            //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
            $linha.= $cdmoeda;

            //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
            $linha.= $cddigito;

            //Valor (Cd. Barra) - Posição: 23 a 36 - Picture: 9(14)
            $linha.= $valor;

            //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
            $linha.= $cdcampo;
            
            //Nome do cedente - Posição: 62 a 91 - Picture: X(30)
            $linha.= Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30));
            
            //Data do vencimento - Posição: 92 a 99 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do Título - Posição: 100 a 114 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Valor do Desconto + Abatimento - Posição: 115 a 129 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0',15);

            //Valor do Acrescimo - Posição: 130 a 144 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0',15);

            //Data do pagamento - Posição: 145 a 152 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do pagto - Posição: 153 a 167 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Quantidade de moeda - Posição:  168 a 182 - Picture: 9(15)
            $linha.= str_repeat('0',15);

            //Cód. Refer. atribuído pelo sacado - Posição: 183 a 202 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                $boleto->getCdfilial(),
                $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);
            
            //Nosso número - Posição: 203 a 222 - Picture: X(20)
            if ($idopecnabpag == 'E'){
                 $linha.= str_pad(substr($boleto->getNrretcnabpag(),0,20),20,' ',STR_PAD_RIGHT);
            } else {
                $linha.= str_repeat(' ', 20); 
            }
            
            //Código de Moeda - Posição: 223 a 224 Picture: X(2)
            $linha .= '09';
            
            //CNAB - Posição: 225 a 230  - Picture: X(6)
            $linha.= str_repeat(' ', 6);
            
            //Ocorrências - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ', 10);
            
            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }

    }
    
    public function geraSegmentoJ52($boleto, $idopecnabpag)
    {
        try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SICREDI;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= 'J';

            //CNAB   posicao 15 a 15 Picture x(1)
            $linha.= str_repeat(' ', 1);
            
            //Tipo de movimento - Posição: 15 a 17 - Picture: 9(3)
            if ($idopecnabpag == 'E')
                $linha.= '99';
            else
                $linha.= '01';
            
            //Identificação de Registro Opcional - Posição: 18 a 19 - Picture: 9(2)
            $linha.= '52';
            
            //Tipo inscricao da Empresa - Posição: 20 a 20 - Picture: 9(1)
            $linha.= $this->tipoinscricao;
            
            //CNPJ empresa debitada - Posição: 21 a 35 - Picture: 9(15)
            $linha.= str_pad(trim($this->inscricaojuridica),15,'0',STR_PAD_LEFT);
            
            //Nome da filial ou Empresa - Posição: 36 a 75 - Picture: X(40)
            $linha.= self::preenche_brancos(substr(trim(strtoupper($this->nomeempresa)),0,40),40);

            //Tipo de Inscrição do Beneficiário - Posição: 76 a 76 - Picture: 9(1)
            $fornecedor = Fornecedor::findOneByNrorgAndCdforneced($this->nrorg, $boleto->getCdforneced());
            if ($fornecedor->getIdtpijurforn() == 'F')
                $linha.= '1';
            else
                $linha.= '2';
            
            //Número de Inscrição do Beneficiário - Posição: 77 a 91 - Picture: 9(15)
            if ($fornecedor->getIdtpijurforn() == 'F'){
                if($boleto->getNrinscnabbop())
                    $linha.= str_pad(
                                substr(trim($boleto->getNrinscnabbop()),0,9) +
                                str_repeat('0', 4) +
                                substr(trim($boleto->getNrinscnabbop()),10,2)
                             , 15,'0',STR_PAD_LEFT);
                else
                    $linha.= str_pad(
                                substr(trim($fornecedor->getNrinsjurforn()),0,9) +
                                str_repeat('0', 4) +
                                substr(trim($fornecedor->getNrinsjurforn()),10,2)
                             , 15,'0',STR_PAD_LEFT);
            }else{
                
                if($boleto->getNrinscnabbop())
                    $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getNrinscnabbop())),0,15), 15,'0',STR_PAD_LEFT);
                else
                    $linha.= str_pad(substr(trim(Util::campoCnabValido($fornecedor->getNrinsjurforn())),0,15), 15,'0',STR_PAD_LEFT);
                    
            }
            
            //Nome do Beneficiário - Posição: 72 a 131 - Picture: x(40)
            $linha.= str_pad(strtoupper(substr($boleto->getForneced()->getNmrazsocforn(), 0, 40)), 40, ' ', STR_PAD_RIGHT);
            
            //CAMPO: Tipo/ Numero de Inscrição do Sacador Avalista - POSIÇÃO: 132 a 147 - PICTURE: 9(16)
            $linha.= str_repeat('0',16);
            
            //Nome do Sacador Avalista - Posição: 148 a 187 - Picture: X(40)
            $linha.= str_repeat(' ',40);
            
            //Brancos - Posição: 188 a 240 - Picture: X(53)
            $linha.= str_repeat(' ',53);
            
            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }

    }

    public function geraSegmentoO($boleto, $idopecnabpag)
    {
        try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SICREDI;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);
            
            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(1)
            $linha.= 'O';
            
            //Tipo de movimento - Posição: 15 a 15 - Picture: 9(01)
            if ($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha .= '9';  // Exclusão
            }
            else{
                $linha.=  '0';  // Inclusão e Bloqueio
                //Somatório dos registros com tipo de movimento = 0 (inclusão)
                $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
            }

            //Código instrução para movimento - Posição: 16 a 17 - Picture: 9(02)
            if ($this->remessa->getIdopecnabpag() == self::INCLUSAO){
                $linha .= '00';  //INCLUSAO
            }
            else{
                if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                    $linha.=  '99';  // EXLUSAO
                }
                else{//BLoqueio
                    $linha.=  '09';
                }
            }
            
            //Código de Barras - Posição: 18 a 61 - Picture: X(44)
            if($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS){
                if($boleto->getCdbarrapag()){
                    $cdbanco = str_pad(substr($boleto->getCdbarrapag(),0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($boleto->getCdbarrapag(),3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($boleto->getCdbarrapag(),4,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($boleto->getCdbarrapag(),5,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($boleto->getCdbarrapag(),19,25),25,'0',STR_PAD_LEFT);
                } else {
                    $linhaBarra = '';
                    $dslinhbarpag = $boleto->getDslinhbarpag();
                    for($i = 0;$i<strlen($dslinhbarpag);$i++){
                        if(in_array($dslinhbarpag[$i],array('0','1','2','3','4','5','6','7','8','9'))){
                            $linhaBarra.= $dslinhbarpag[$i];
                        }
                    }
                    $cdbanco = str_pad(substr($linhaBarra,0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($linhaBarra,3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($linhaBarra,32,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($linhaBarra,33,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($linhaBarra,4,25),25,'0',STR_PAD_LEFT);
                }
            }
            //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
            $linha.= $cdbanco;

            //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
            $linha.= $cdmoeda;

            //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
            $linha.= $cddigito;

            //Valor (Cd. Barra) - Posição: 23 a 36 - Picture: 9(14)
            $linha.= $valor;

            //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
            $linha.= $cdcampo;
            
            //Nome da Concessionária / Órgão - Posição: 62 a 91 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);

            //Data do vencimento - Posição: 92 a 99 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Campo: Data do Pagamento - Posição: 100 a 107 - Picture: X(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            //Valor do Pagamento - Posição: 108 a 122 - Picture: X(15)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);
            
            //Nº do Docto Atribuído pela Empresa - Posição: 123 a 142 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);

            //Nº do Docto Atribuído pelo Banco - Posição: 143 a 162 - Picture: X(20)
            $linha.= str_repeat(' ',20);

            //Uso Exclusivo FEBRABAN/CNAB - Posição: 163. a 230 - Picture: X(68)
            $linha.= str_repeat(' ', 68);

            //Ocorrências - Posição: 231 a 240 - Picture: X(21)
            $linha.= str_repeat(' ', 10);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraSegmentoA($boleto, $idopecnabpag)
    {
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SICREDI;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            $linha.= 'A';
            
            //Tipo de movimento - Posição: 15 a 17 - Picture: 9(3)
            if ($idopecnabpag == 'E')
                $linha.= '999';
            else
                $linha.= '000';
                
            //Complemento de registro - Posição: 18 a 20 - Picture: 9(3)
            $linha.= '000';
            
            if($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG){
                
                //Código do banco do favorecido - Posição: 21 a 23 - Picture: 9(03)
                $linha.= str_pad(substr(trim($this->contcorr->getCdbanccnab()),0,3),3,'0',STR_PAD_LEFT);
                
                //Número da agência mantenedora - Posição: 24 a 28 - Picture: 9(05)
                $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);

                //Dídito verificador da agência - Posição: 29 a 29 - Picture: X(01)
                $linha.= ' ';

                //Número da conta - Posição: 30 a 41 - Picture: 9(12)
                $linha.= self::preenche_brancos('',12);

                //Dídito verificador da conta - Posição: 42 a 42 - Picture: X(01)
                $linha.= '0';

                //Dídito verificador da agência/conta - Posição: 43 a 43 - Picture: X(01)
                $linha.= ' ';
            }
            else{
                
                //Código do banco do favorecido - Posição: 21 a 23 - Picture: 9(03)
                $linha.= str_pad(substr(trim($boleto->getCdbancbolp()),0,3),3,'0',STR_PAD_LEFT);
                
                //Busca a máscara da CONTA e AGÊNCIA do fornecedor cadastrada na tela de Banco
                /** @var $bancoFornecedor Banco */
                $bancoFornecedor = self::getBancoForn($boleto->getCdbancbolp(), $boleto->getNrorg());

                //{========= Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado com as máscaras,
                // entrará nesse if =========}
                if($bancoFornecedor and $bancoFornecedor->getCdpictagen()){
                    //{ se na máscara da agência NÃO tem '-' }
                    if(strpos($bancoFornecedor->getCdpictagen(),'-') === false){
            
                        //Agencia
                        $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,5)
                            ,5,'0',STR_PAD_LEFT);
                        //DV
                        $linha.= ' ';
                    }
                    else{
                        
                        //{ Caso a máscara da agência TENHA '-' }
                        //Agencia
                        $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,
                                strpos($bancoFornecedor->getCdpictagen(),'-'))
                            ,5,'0',STR_PAD_LEFT);

                        //Se foi cadastrada a agencia do favorecido como 9999 na tela de boleto e
                        //tempos depois foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                        //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema.
                        if(strlen(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                ,strpos($bancoFornecedor->getCdpictagen(),'-')))>0){

                            //DV
                            $pos = strpos($bancoFornecedor->getCdpictagen(),'-');
                            $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                ,$pos,1);
                        }
                        else{
                            $linha.= ' ';
                        }
                    }
                }
                else{
                    //{======== Se o Banco do fornecedor NÃO estiver cadastrado no sistema,
                    //NÃO parametrizado com as máscaras, entrará nesse else ====}
                    if(strpos($boleto->getCdagenbolp(),'-')=== false){
                        // { se o que foi digitado no campo de agencia do favorecido da tela de boleto NÃO tem '-' }
                        //Agencia
                        $linha.= str_pad(substr(trim($boleto->getCdagenbolp()),0,5),5,'0',STR_PAD_LEFT);

                        //DV
                        $linha.= ' ';
                    }
                    else{
                        //{ caso o que foi digitado no campo de agencia do favorecido da tela de boleto TENHA '-' }
                        //Agencia
                        $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdagenbolp(),0,
                            strpos($boleto->getCdagenbolp(),'-')
                        )),5,'0',STR_PAD_LEFT);

                        //DV
                        $pos = strpos($boleto->getCdagenbolp(),'-') +1;
                        $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                            ,$pos,1);
                    }
                }

                //Número da conta corrente favorecido - Posição: 030 a 041 - Picture: 9(11)
                //MAIS
                //Dídito verificador da conta - Posição: 42 a 42 - Picture: X(01)

                //{========== Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado
                //com as máscaras, entrará nesse if ===========}
                
                if($bancoFornecedor and $bancoFornecedor->getCdpictcont()){
                    //{ se na máscara da conta corrente TEM '-' }
                    if(strpos($bancoFornecedor->getCdpictcont(),'-')){

                        //{ tamDifVer só pode assumir os valor 1 ou 2.
                        // Assume 1 quando a máscara é [QNT_AQUI_N_IMPORTA]-9 e
                        // assume 2 quando é [QNT_AQUI_N_IMPORTA]-99 }

                        $tamDigVer = strlen(trim(substr($bancoFornecedor->getCdpictcont(),
                            strpos($bancoFornecedor->getCdpictcont(),'-')+1,
                            2)));
                        if($tamDigVer>1){
                            //CC
                            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                    strpos($bancoFornecedor->getCdpictcont(),'-'))),
                                12,'0',STR_PAD_LEFT);

                            $linha.= ' ';

                            //Se foi cadastrada a cc do favorecido como 9999 na tela de boleto e tempos depois
                            //foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                            //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema
                            if(strlen(substr(trim(Util::campoCnabValido($boleto->getCdccorbolp()))
                                    ,strpos($bancoFornecedor->getCdpictcont(),'-')))>0){
                                //DV
                                $pos = strpos($bancoFornecedor->getCdpictcont(),'-')+1 ;
                                $linha.= substr(trim(Util::campoCnabValido($boleto->getCdccorbolp()))
                                    ,$pos,1);
                            }
                            else{
                                $linha.= ' ';
                            }

                        }
                        else{
                            //CC
                            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                -1
                            )),12,'0',STR_PAD_LEFT);
                            
                            $linha.= ' ';
                            
                            //DV
                            $linha.= substr(trim(Util::campoCnabValido($boleto->getCdccorbolp())),
                                -1);

                        }

                    }
                    else{
                        
                        //{ CASO a máscara da conta corrente NÃO tenha '-' }
                        if($boleto->getCdbancbolp() == self::CODIGO_COMPENSACAO_SICREDI){
                            //CC
                            $linha.= str_pad(trim(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,-1)
                            )),12,'0',STR_PAD_LEFT);
                            
                            $linha.= ' ';

                            //DV
                            $linha.= substr($boleto->getCdccorbolp(),-1);

                        }
                        else{
                            //
                            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,12)
                            ),12,'0',STR_PAD_LEFT);

                            $linha.= ' ';

                            //DV
                            $linha.= ' ';
                        }
                    }

                }
                // {======== Se o Banco do fornecedor NÃO estiver cadastrado no sistema, NÃO parametrizado com
                //as máscaras, entrará nesse else ====}
                else{
                    if(strpos($boleto->getCdccorbolp(),'-')){
                        //se o que foi digitado no campo de conta Corrente do favorecido da tela de boleto tem '-'
                        //CC

                        //{ tamDifVer só pode assumir os valor 1 ou 2.
                        // Assume 1 quando a máscara é [QNT_AQUI_N_IMPORTA]-9 e
                        // assume 2 quando é [QNT_AQUI_N_IMPORTA]-99 }

                        $tamDigVer = strlen(trim(substr($boleto->getCdccorbolp(),
                            strpos($boleto->getCdccorbolp(),'-')+1,
                            2)));

                        if($tamDigVer>1){
                            //CC
                            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                    strpos($boleto->getCdccorbolp(),'-'))
                            ),11,'0',STR_PAD_LEFT);
                            $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                strpos($boleto->getCdccorbolp(),'-')+1,1));
                                
                            $linha.= ' ';
                            
                            //DV
                            $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                strpos($boleto->getCdccorbolp(),'-')+2,1));
                        }
                        else{
                            //CC
                            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                    strpos($boleto->getCdccorbolp(),'-'))
                            ),12,'0',STR_PAD_LEFT);

                            $linha.= ' ';

                            //DV
                            $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                strpos($boleto->getCdccorbolp(),'-')+1));
                        }
                    }
                    //CASO o que foi digitado no campo de conta corrente do favorecido da tela de
                    //Boleto NÃO tenha '-'
                    else{
                        if($boleto->getCdbancbolp() == self::CODIGO_COMPENSACAO_SICREDI){
                            //CC
                            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,-1)
                            ),12,'0',STR_PAD_LEFT);

                            $linha.= ' ';

                            //DV
                            $linha.= substr($boleto->getCdccorbolp(),-1);
                        }
                        else{
                            //CC
                            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,12)
                            ),12,'0',STR_PAD_LEFT);

                            $linha.= ' ';

                            //DV
                            $linha.= ' ';
                        }
                    }
                }
            }
                
            
            //Nome do favorecido - Posição: 44 a 73 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);
            
            //Seu número - Posição: 74 a 93 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                $boleto->getCdfilial(),
                $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);
            
            //Data do pagamento - Posição: 094 a 101 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            //Campo: Tipo Moeda - Posição: 102 a 104 - Picture: X(03)
            $linha.= 'BRL';
            
            //Zeros - Posição: 105 a 119 - Picture: X(15)
            $linha.= str_repeat('0', 15);
            
            //Valor do pagto - Posição: 120 a 134 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);
            
            //Nosso número - Posição: 135 a 154 - Picture: X(20)
            if ($idopecnabpag == 'E'){
                 $linha.= str_pad(substr($boleto->getNrretcnabpag(),0,20),20,' ',STR_PAD_RIGHT);
            } else {
                $linha.= str_repeat(' ', 20); 
            }

            //Data efetiva - Posição: 155 a 162 - Picture: 9(8)
            $linha.= str_repeat('0', 8);

            //Valor efetiva - Posição: 163 a 177 - Picture: 9(15)
            $linha.= str_repeat('0', 15);

            //Informação 2 Detalhe - Posição: 178 a 217 - Picture: X(40)
            $linha.= str_repeat(' ', 40);
            
            //Código finalidade ted/DOC - Posição: 218 a 219 Doc - Posição: 220 a 224 Picture: X(5)
            //$formaPagto = self::formaPagamento($cdbanco, $boletos);
            if($this->formapgto == '03'){
                $linha.= '07';
                $linha.= str_repeat(' ', 5);
            } else if($this->formapgto == '41'){
                $linha.= str_repeat(' ', 2);
                $linha.= '00005';
            } else {
              $linha.= str_repeat(' ', 7);  
            }
            
            //Código finalidade complementar - Brancos - Posição: 225 a 226
            $linha.= str_repeat(' ', 2);
            
            // CNAB - Brancos - Posição: 227 a 229
            $linha.= str_repeat(' ', 3);
            
            //Aviso - Posição: 230 a 230 - Picture: X(1)
            $linha.= '0';
            
            //Ocorrências - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ', 10);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraSegmentoN($boleto, $idopecnabpag)
    {
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SICREDI;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);
            
            ///Código de segmento do reg. detalhe - Posição: 14 a 14 - Picture: 9(1)
            $linha.= 'N';
            
             //Tipo de movimento - Posição: 15 a 15 - Picture: 9(01)
            if ($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                $linha .= '9';  // Exclusão
            }
            else{
                $linha.=  '0';  // Inclusão e Bloqueio
                //Somatório dos registros com tipo de movimento = 0 (inclusão)
                $this->vrPagtLote += round($boleto->getVrfinalbpag(),2);
            }

            //Código instrução para movimento - Posição: 16 a 17 - Picture: 9(02)
            if ($this->remessa->getIdopecnabpag() == self::INCLUSAO){
                $linha .= '00';  //INCLUSAO
            }
            else{
                if($this->remessa->getIdopecnabpag() == self::EXCLUSAO){
                    $linha.=  '99';  // EXLUSAO
                }
                else{//BLoqueio
                    $linha.=  '09';
                }
            }
                
            //Identificação do tributo - Posição: 18 a 19 - Picture: 9(02)
            if ($this->formapgto == '16')
                $linha = self::darfNormal($linha, $boleto);
            elseif($this->formapgto == '17')
                   $linha = self::GPS($linha, $boleto);
                elseif ($this->formapgto == '18')
                    $linha =self::darfSimples($linha, $boleto);
                elseif($this->formapgto == '22')
                    $linha =self::GARE($linha, $boleto);
                elseif($this->formapgto == '21')
                    $linha =self::DARJ($linha, $boleto);
                else
                    $linha = self::FGTS($linha, $boleto);

            //Seu número - Posição: 196 a 215 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),$boleto->getCdfilial(),$boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);
            
            //Nosso Número número - Posição: 216 a 230 - Picture: X(15)
            $linha.= str_repeat(' ',15);
            
            //Ocorrências - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);
            
            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function darfNormal( $linha, $boleto)
    {
        //Identificação do tributo - Posição: 18 a 19 - Picture: 9(02)
        $linha.= '02'; //DARF  Normal

        //Código da receita - Posição: 20 a 23 - Picture: 9(04)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if (!Empty($cdreceitaguia))
            $linha.= str_pad(trim(is_numeric($cdreceitaguia)), 4, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 4);

        //Tipo inscrição empresa debitada - Posição: 24 a 24 - Picture: 9(1)
        $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
        
        if ($empresa->getIdtpijurempr() == 'F')
            $linha.= '1'; //CPF
        else
            $linha.= '2'; //CNPJ

        //CPF ou CNPJ do contribuinte  - Posição: 25 a 38 - Picture: 9(14)
        $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);

        //Período de Apuração - Posição: 39 a 46 - Picture: 9(08)
        if ($boleto->getDtapuracguia() != 0)
            $linha.= self::formataData($boleto->getDtapuracguia());
        else
            $linha.= str_repeat('0', 8);

        //Número de referência - Posição: 47 a 63 - Picture: 9(17)
        $nrrefeguia = $boleto->getNrrefeguia();
        if (!Empty($nrrefeguia))
            $linha.= str_pad(trim(is_numeric($nrrefeguia), 17), 17, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 17);
        
        //Valor Principal - Posição: 64 a 77 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),14,'0',STR_PAD_LEFT);

        //Valor da Multa - Posição: 78 a 91 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),14,'0',STR_PAD_LEFT);

        //Valor dos Juros/ Encargos - Posição: 92 a 105 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Valor total a ser pago - Posição: 106 a 119 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Data de Vencimento - Posição: 120 a 127 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');

        //Data do Pagamento - Posição: 128 a 135 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');

        //Brancos - Posição: 136 a 165 - Picture: X(30)
        $linha.= str_repeat(' ', 30);
        
        //Nome do Contribuinte - Posição: 166 a 195 - Picture: 9(30)
        $linha.= str_pad(substr(strtoupper($this->nomeempresa), 0, 30), 30, ' ');
        
        return $linha;
    }

    public function darfSimples($linha, $boleto)
    {
        //Identificação do tributo - Posição: 18 a 19 - Picture: 9(02)
        $linha.= '03'; //DARF Simples
        
        //Código da receita - Posição: 20 a 23 - Picture: 9(04)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if (!Empty($cdreceitaguia))
            $linha.= str_pad(trim(is_numeric($cdreceitaguia)), 4, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 4);
        
        //Tipo inscrição empresa debitada -Posição: 24 a 24 - Picture: 9(1)
        $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
        if ($empresa->getIdtpijurempr() == 'F')
            $linha.= '1'; //CPF
        else
            $linha.= '2'; //CNPJ
        
        //CPF ou CNPJ do contribuinte  - Posição: 25 a 38 - Picture: 9(14)
        $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);
        
        //Período de Apuração - Posição: 39 a 46 - Picture: 9(08)
        if ($boleto->getDtapuracguia() != 0)
            $linha.= $boleto->getDtapuracguia()->format('dmY');
        else
            $linha.= str_repeat('0', 8);
        
        //Valor da Receita Bruta Acumulada - Posição: 47 a 55 - Picture: 9(7)V2(02)
        $linha.= str_pad(Util::campoCnabValido($boleto->getVroutrentguia()),9, STR_PAD_LEFT);
        
        //Percentual - Posição: 56 a 59 - Picture: 9(2)V2(02)
        $linha.= str_pad(Util::campoCnabValido($boleto->getVrpercenttrib()),4, STR_PAD_LEFT);
        
        //Brancos - Posição: 60 - 63 - Picture: X(04)
        $linha.= str_repeat(' ', 4);
        
        //Valor Principal - Posição: 64 a 77 - Picture: 9(12)V2(02)
        //É O RESULTADO DA MULTIPLICAÇÃO: VALOR DO PERCENTUAL X RECEITA BRUTA ACUMULADA
        if($boleto->getVrpercenttrib() != 0 && $boleto->getVroutrentguia() != 0 ){
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia() * $boleto->getVrpercenttrib(),2)),14,STR_PAD_LEFT);
        } else {
             $linha .= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),14,STR_PAD_LEFT);
        }
        
        //Valor da Multa - Posição: 78 a 91 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),14,STR_PAD_LEFT);
        
        //Valor dos Juros/ Encargos - Posição: 92 a 105 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),14,STR_PAD_LEFT);
        
        //Valor total a ser pago - Posição: 106 a 119 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),14,STR_PAD_LEFT);
        
        //Data de Vencimento - Posição: 120 a 127 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Data do Pagamento - Posição: 128 a 135 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Brancos - Posição: 136 a 165 - Picture: X(30)
        $linha.= str_repeat(' ', 30);

        //Nome do Contribuinte - Posição: 166 a 195 - Picture: 9(30)
        $linha.= str_pad(substr(strtoupper($this->nomeempresa), 0, 30), 30, ' ');
        
        return $linha;
    }

    public function GPS($linha, $boleto)
    {
        //Identificação do tributo - Posição: 18 a 19 - Picture: 9(02)
        $linha.= '01'; //GPS
        
        //Código da receita - Posição: 20 a 23 - Picture: 9(04)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if (!Empty($cdreceitaguia))
            $linha.= str_pad(trim(is_numeric($cdreceitaguia)), 4, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 4);
        
        //Mês e ano da Competência - Posição: 24 a 29 - Picture: 9(06)
        if ($boleto->getDtapuracguia() != 0)
            $linha.= $boleto->getDtapuracguia()->format('mY');
        else
            $linha.= str_repeat('0', 6);
        
        //Identificador - Posição: 30 a 43 - Picture: 9(14)
        $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);
        
        //Valor previsto do pagamento do INSS - Posição: 44 a 57 - Picture: 9(12) V9(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag() - $boleto->getVroutrentguia() + $boleto->getVrmultabpag() + $boleto->getVrmordiabpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Valor de outras entidades -  Posição: 58 a 71 - Picture: 9(12) V9(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia(),2)),14,'0',STR_PAD_LEFT);
        
        //Atualização Monetária - Posição: 72 a 85 - Picture: 9(12) V9(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag() + $boleto->getVrmordiabpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Valor Arrecadado - Posição: 86 a 99 - Picture: 9(12) V9(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Data da Arrecadação/Efetivação do Pagamento - Posição: 100 a 107 - Picture: 9(12) V9(02)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //COMPLEMENTO DO REGISTRO - Posição: 108 a 115 - Picture: X(08)
        $linha.= str_repeat(' ', 8);
        
        //INFORMAÇÕES COMPLEMENTARES - Posição: 116 a 165 - Picture: X(50)
        //USO EMPRESA
        $linha.= str_repeat(' ', 50);
        
        //Nome do Contribuinte - Posição: 166 a 195 - Picture: 9(30)
        $linha.= str_pad(substr(strtoupper($this->nomeempresa), 0, 30), 30, ' ');
        
        return $linha;
    }
    
    public function GARE($linha, $boleto){
        
        //Identificação do tributo - Posição: 18 a 19 - Picture: 9(02)
        $linha.= '05'; //GARE ICMS SP
        
        //Código da receita - Posição: 20 a 23 - Picture: 9(04)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if (!Empty($cdreceitaguia))
            $linha.= str_pad(trim(is_numeric($cdreceitaguia)), 4, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 4);
        
        //Tipo inscrição empresa debitada - Posição: 24 a 24 - Picture: 9(1)
        $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
        if ($empresa->getIdtpijurempr() == 'F')
            $linha.= '1'; //CPF
        else
            $linha.= '2'; //CNPJ
        
        //CPF ou CNPJ do contribuinte  - Posição: 25 a 38 - Picture: 9(14)
        $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);
        
        //Inscrição Estadual - Posição: 39 a 50 - Picture: 9(08)
        $linha.= str_pad($empresa->getCdinscesta(),12, '0', STR_PAD_LEFT);

        //Divida Ativa / Número Etiqueta - Posição: 51 a 63 - Picture: 9(13)
        $nrrefeguia = $boleto->getNrrefeguia();
        if (!Empty($nrrefeguia))
            $linha.= str_pad(trim(is_numeric($nrrefeguia)), 13, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 13);
        
        //Mês e Ano de Referência - Posição: 64 a 69 - Picture: 9(06)
        if ($boleto->getDtapuracguia() != 0)
            $linha.= self::formataData($boleto->getDtapuracguia());
        else
            $linha.= str_repeat('0', 6);
        
        //Número Parcela/Notificação - Posição: 70 a 82 - Picture: 9(13)
        $linha.= str_pad($boleto->getNrparcbolpag(),13, '0', STR_PAD_LEFT);
        
        //Valor da Receita - Posição: 83 a 96 - Picture: 9(7)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVroutrentguia(),2)),14,'0',STR_PAD_LEFT);
        
        //Valor dos Juros/ Encargos - Posição: 97 a 110 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Valor da Multa - Posição: 111 a 124 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Valor total a ser pago - Posição: 125 a 138 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Data de Vencimento - Posição: 139 a 146 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Data do Pagamento - Posição: 147 a 154 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Brancos - Posição: 155 a 165 - Picture: X(11)
        $linha.= str_repeat(' ', 11);
        
        //Nome do Contribuinte - Posição: 166 a 195 - Picture: 9(30)
        $linha.= str_pad(substr(strtoupper($this->nomeempresa), 0, 30), 30, ' ');

        return $linha;
        
    }
    
    
    public function DARJ($linha, $boleto){
        //Identificação do tributo - Posição: 18 a 19 - Picture: 9(02)
        $linha.= '04'; //DARJ
        
        //Código da receita - Posição: 20 a 23 - Picture: 9(04)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        
        if (!Empty($cdreceitaguia))
            $linha.= str_pad(trim(is_numeric($cdreceitaguia)), 4, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 4);
            
        //Tipo inscrição empresa debitada - Posição: 24 a 24 - Picture: 9(1)
        
        $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
        if ($empresa->getIdtpijurempr() == 'F')
            $linha.= '1'; //CPF
        else
            $linha.= '2'; //CNPJ
        
        //CPF ou CNPJ do contribuinte - Posição: 25 a 38 - Picture: 9(14)
        $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);
        
        //Inscrição Estadual - Posição: 39 a 46 - Picture: 9(08)
        $linha.= str_pad(substr($empresa->getCdinscesta(),0,8),8, '0', STR_PAD_LEFT);
        
        //Número Documento Origem - Posição: 47 a 62 - Picture: 9(16)
        $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(), $boleto->getCdfilial(), $boleto->getNrorg());
        $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),16,'0',STR_PAD_LEFT);
        
        //Complemento de Registro - Posição: 63 a 63 - Picture: X(01)
        $linha.= str_repeat(' ', 1);
        
        //Valor Principal - Posição: 64 a 77 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag() - $boleto->getVrmordiabpag() + $boleto->getVrmultabpag(),2)),14,STR_PAD_LEFT);
        
        //Valor Atualização Monetária - Posição: 78 a 91 - Picture: 9(12)V2(02)
        $linha.= str_repeat('0', 14);
        
        //Valor dos Juros/ Encargos - Posição: 92 a 105 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmordiabpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Valor da Multa - Posição: 106 a 119 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrmultabpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Valor total a Recolher - Posição: 120 a 133 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Data de Vencimento - Posição: 134 a 141 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Data do Pagamento - Posição: 142 a 149 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Período de Referência - Posição: 150 a 155 - Picture: 9(06)
        if ($boleto->getDtapuracguia() != 0)
            $linha.= $boleto->getDtvenbolpag()->format('mY');
        else
            $linha.= str_repeat('0', 6);
        
        //Brancos - Posição: 155 a 165 - Picture: X(11)
        $linha.= str_repeat(' ', 10);
        
        //Nome do Contribuinte - Posição: 166 a 195 - Picture: 9(30)
        $linha.= str_pad(substr(strtoupper($this->nomeempresa), 0, 30), 30, ' ');
       
        return $linha;
    }
    
    public function FGTS($linha, $boleto){
        
        //Identificação do tributo - Posição: 18 a 19 - Picture: 9(02)
        $linha.= '11'; //FGTS
        
        //Código da RECEITA - Posição: 20 a 23 - Picture: 9(04)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        
        if (!Empty($cdreceitaguia))
            $linha.= str_pad(trim(is_numeric($cdreceitaguia)), 4, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 4);
            
        
        //TIPO DE INSCRIÇÃO DO CONTRIBUINTE - Posição: 24 a 24 - Picture: 9(01)
        $empresa = empresa::findOneByNrorgAndCdempresa($this->nrorg, $this->remessa->getCdempresa());
        if ($empresa->getIdtpijurempr() == 'F')
            $linha.= '1'; //CPF
        else
            $linha.= '2'; //CNPJ
        
        //CPF ou CNPJ do contribuinte  - Posição: 25 a 38 - Picture: 9(14)
        $linha.= str_pad(substr(trim(Util::campoCnabValido($this->empresa->getNrinsjurempr())),0,14), 14,'0',STR_PAD_LEFT);
        
        //CODIGO DE BARRAS - Posição:  39 - 86 - Picture: X(48)
        if($boleto->getCdbarrapag()){
            $linha .= str_pad($boleto->getCdbarrapag(),48, ' ',STR_PAD_LEFT);
        } else {
           $linha.= str_repeat(' ', 48);
        }
        
        //IDENTIFICADOR DO FGTS - Posição: 087 - 102 - Picture: 9(16)
        $nrrefeguia = $boleto->getNrrefeguia();
        
        if (!Empty($nrrefeguia))
            $linha.= str_pad(trim(is_numeric($nrrefeguia)), 16, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 16);
        
        //LACRE DE CONECTIVIDADE SOCIAL - Posição: 103 - 111 - Pictures: 9(09)
        $linha.= str_repeat('0', 9);
        
        //DIGITO DO LACRE DE CONECTIVIDADE SOC. - Posição: 112 - 113 -Picture: 9(02)
        $linha.= str_repeat('0', 2);
        
        //Nome do Contribuinte - Posição: 114 a 143 - Picture: 9(30)
        $linha.= str_pad(substr(strtoupper($this->nomeempresa), 0, 30), 30, ' ');
        
        //Data do Pagamento - Posição: 144 a 151 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');
        
        //Valor total a ser pago - Posição: 152 a 165 - Picture: 9(12)V2(02)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),14,'0',STR_PAD_LEFT);
        
        //Brancos - Posição: 166 a 195 - Picture: X(30)
        $linha.= str_repeat(' ', 30);
        
        return $linha;
    }
    
    public static function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
    public function geraSegmentoAPIX($boleto, $idopecnabpag)
    {
        try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SICREDI;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= 'A';

            //Tipo de movimento - Posição: 15 a 17 - Picture: 9(3)
            if ($idopecnabpag == 'E')
                $linha.= '999';
            else
                $linha.= '000';

            $cdbanco   = '000';                 //PARA PIX PREENCHER COM ZEROS
            $cdmoeda   = '0';                   //PARA PIX PREENCHER COM ZEROS
            $cddigito  = '0';                   //PARA PIX PREENCHER COM ZEROS
            $fatorvenc = '0000';                //PARA PIX PREENCHER COM ZEROS
            $valor     = str_repeat('0',10);    //PARA PIX PREENCHER COM ZEROS
            $cdcampo   = str_repeat('0',25);    //PARA PIX PREENCHER COM ZEROS
            
            //Complemento de registro - Posição: 18 a 20 - Picture: 9(3)  - PIX (SPI)
            $comppix = '009';
            $linha.= $comppix;

            //Agência e Conta do Fornecedor
            //Posição: 24 a 43
            if (!$boleto->getIdchavepix()){
                if($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG){
                    
                    //Código do banco do favorecido - Posição: 21 a 23 - Picture: 9(03)
                    $linha.= str_pad(substr(trim($this->contcorr->getCdbanccnab()),0,3),3,'0',STR_PAD_LEFT);
                    
                    //Número da agência mantenedora - Posição: 24 a 28 - Picture: 9(05)
                    $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,5),5,'0',STR_PAD_LEFT);
    
                    //Dídito verificador da agência - Posição: 29 a 29 - Picture: X(01)
                    $linha.= ' ';
    
                    //Número da conta - Posição: 30 a 41 - Picture: 9(12)
                    $linha.= self::preenche_brancos('',12);
    
                    //Dídito verificador da conta - Posição: 42 a 42 - Picture: X(01)
                    $linha.= '0';
    
                    //Dídito verificador da agência/conta - Posição: 43 a 43 - Picture: X(01)
                    $linha.= ' ';
                }else{
                    
                    //Código do banco do favorecido - Posição: 21 a 23 - Picture: 9(03)
                    $linha.= str_pad(substr(trim($boleto->getCdbancbolp()),0,3),3,'0',STR_PAD_LEFT);
                    
                    //Busca a máscara da CONTA e AGÊNCIA do fornecedor cadastrada na tela de Banco
                    /** @var $bancoFornecedor Banco */
                    $bancoFornecedor = self::getBancoForn($boleto->getCdbancbolp(), $boleto->getNrorg());
    
                    //{========= Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado com as máscaras,
                    // entrará nesse if =========}
                    if($bancoFornecedor and $bancoFornecedor->getCdpictagen()){
                        //{ se na máscara da agência NÃO tem '-' }
                        if(strpos($bancoFornecedor->getCdpictagen(),'-') === false){
                
                            //Agencia
                            $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,5)
                                ,5,'0',STR_PAD_LEFT);
                            //DV
                            $linha.= ' ';
                        }
                        else{
                            
                            //{ Caso a máscara da agência TENHA '-' }
                            //Agencia
                            $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp())),0,
                                    strpos($bancoFornecedor->getCdpictagen(),'-'))
                                ,5,'0',STR_PAD_LEFT);
    
                            //Se foi cadastrada a agencia do favorecido como 9999 na tela de boleto e
                            //tempos depois foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                            //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema.
                            if(strlen(substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                    ,strpos($bancoFornecedor->getCdpictagen(),'-')))>0){
    
                                //DV
                                $pos = strpos($bancoFornecedor->getCdpictagen(),'-');
                                $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                    ,$pos,1);
                            }
                            else{
                                $linha.= ' ';
                            }
                        }
                    }
                    else{
                        //{======== Se o Banco do fornecedor NÃO estiver cadastrado no sistema,
                        //NÃO parametrizado com as máscaras, entrará nesse else ====}
                        if(strpos($boleto->getCdagenbolp(),'-')=== false){
                            // { se o que foi digitado no campo de agencia do favorecido da tela de boleto NÃO tem '-' }
                            //Agencia
                            $linha.= str_pad(substr(trim($boleto->getCdagenbolp()),0,5),5,'0',STR_PAD_LEFT);
    
                            //DV
                            $linha.= ' ';
                        }
                        else{
                            //{ caso o que foi digitado no campo de agencia do favorecido da tela de boleto TENHA '-' }
                            //Agencia
                            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdagenbolp(),0,
                                strpos($boleto->getCdagenbolp(),'-')
                            )),5,'0',STR_PAD_LEFT);
    
                            //DV
                            $pos = strpos($boleto->getCdagenbolp(),'-') +1;
                            $linha.= substr(trim(Util::campoCnabValido($boleto->getCdagenbolp()))
                                ,$pos,1);
                        }
                    }
    
                    //Número da conta corrente favorecido - Posição: 030 a 041 - Picture: 9(11)
                    //MAIS
                    //Dídito verificador da conta - Posição: 42 a 42 - Picture: X(01)
    
                    //{========== Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado
                    //com as máscaras, entrará nesse if ===========}
                    if($bancoFornecedor and $bancoFornecedor->getCdpictcont()){
                        //{ se na máscara da conta corrente TEM '-' }
                        if(strpos($bancoFornecedor->getCdpictcont(),'-')){
    
                            //{ tamDifVer só pode assumir os valor 1 ou 2.
                            // Assume 1 quando a máscara é [QNT_AQUI_N_IMPORTA]-9 e
                            // assume 2 quando é [QNT_AQUI_N_IMPORTA]-99 }
    
                            $tamDigVer = strlen(trim(substr($bancoFornecedor->getCdpictcont(),
                                strpos($bancoFornecedor->getCdpictcont(),'-')+1,
                                2)));
                            if($tamDigVer>1){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                        strpos($bancoFornecedor->getCdpictcont(),'-'))),
                                    12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //Se foi cadastrada a cc do favorecido como 9999 na tela de boleto e tempos depois
                                //foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                                //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema
                                if(strlen(substr(trim(Util::campoCnabValido($boleto->getCdccorbolp()))
                                        ,strpos($bancoFornecedor->getCdpictcont(),'-')))>0){
                                    //DV
                                    $pos = strpos($bancoFornecedor->getCdpictcont(),'-')+1 ;
                                    $linha.= substr(trim(Util::campoCnabValido($boleto->getCdccorbolp()))
                                        ,$pos,1);
                                }
                                else{
                                    $linha.= ' ';
                                }
    
                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                    -1
                                )),12,'0',STR_PAD_LEFT);
                                
                                $linha.= ' ';
                                
                                //DV
                                $linha.= substr(trim(Util::campoCnabValido($boleto->getCdccorbolp())),
                                    -1);
    
                            }
    
                        }
                        else{
                            //{ CASO a máscara da conta corrente NÃO tenha '-' }
                            if($boleto->getCdbancbolp() == self::CODIGO_COMPENSACAO_SANTANDER){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,-1)
                                ),12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //DV
                                $linha.= substr($boleto->getCdccorbolp(),-1);
    
                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,12)
                                ),12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //DV
                                $linha.= ' ';
                            }
                        }
    
                    }
                    // {======== Se o Banco do fornecedor NÃO estiver cadastrado no sistema, NÃO parametrizado com
                    //as máscaras, entrará nesse else ====}
                    else{
                        if(strpos($boleto->getCdccorbolp(),'-')){
                            //se o que foi digitado no campo de conta Corrente do favorecido da tela de boleto tem '-'
                            //CC
    
                            //{ tamDifVer só pode assumir os valor 1 ou 2.
                            // Assume 1 quando a máscara é [QNT_AQUI_N_IMPORTA]-9 e
                            // assume 2 quando é [QNT_AQUI_N_IMPORTA]-99 }
    
                            $tamDigVer = strlen(trim(substr($boleto->getCdccorbolp(),
                                strpos($boleto->getCdccorbolp(),'-')+1,
                                2)));
    
                            if($tamDigVer>1){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                        strpos($boleto->getCdccorbolp(),'-'))
                                ),11,'0',STR_PAD_LEFT);
                                $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                    strpos($boleto->getCdccorbolp(),'-')+1,1));
                                    
                                $linha.= ' ';
                                
                                //DV
                                $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                    strpos($boleto->getCdccorbolp(),'-')+2,1));
                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,
                                        strpos($boleto->getCdccorbolp(),'-'))
                                ),12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //DV
                                $linha.= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                    strpos($boleto->getCdccorbolp(),'-')+1));
                            }
                        }
                        //CASO o que foi digitado no campo de conta corrente do favorecido da tela de
                        //Boleto NÃO tenha '-'
                        else{
                            if($boleto->getCdbancbolp() == self::CODIGO_COMPENSACAO_SANTANDER){
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,-1)
                                ),12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //DV
                                $linha.= substr($boleto->getCdccorbolp(),-1);
                            }
                            else{
                                //CC
                                $linha.= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(),0,12)
                                ),12,'0',STR_PAD_LEFT);
    
                                $linha.= ' ';
    
                                //DV
                                $linha.= ' ';
                            }
                        }
                    }
                }
            }else{
                $linha.= str_repeat(' ', 20);
            }
            
            //Nome do cedente - Posição: 44 a 73 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);
            
            //Cód. Refer. atribuído pelo sacado - Posição: 74 a 93 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                $boleto->getCdfilial(),
                $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);

            //Data do vencimento - Posição: 94 a 101 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            
            //Tipo da Moeda - Posição: 102 a 104 - Picture: X(3)
            $linha.= 'REA';
            
            //BRANCOS - Posição: 105 a 112 - Picture: X(8)  CÓDIGO ISPB
            $linha.= str_repeat(' ',8);
            
            //- Posição: 113 a 114 - Picture: x(2) IDENTI. TRANSFERENCIA
            if ($boleto->getIdchavepix()){
                $gerabpix = true;
                $linha.= '04';  //04-Chave de Endereçamento caso a chave pix esteja preenchida
            }else{
                $linha.= '01';  //01-Conta  corrente caso a chave pix esteja vazia
            }
            
            //ZEROS- Posição: 115 a 119 - Picture: 9(5) Complemento do Registro
            $linha.= str_repeat('0',5);

            //Valor do Título - Posição: 120 a 134 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Nosso número - Posição: 135 a 149 - Picture: X(15)
            $linha.= str_repeat(' ', 15);

            //Brancos - Posição: 150 a 154 - Picture: X(5)
            $linha.= str_repeat(' ', 5);
            
            //Data efetiva - Posição: 155 a 162 - Picture: 9(8)
            $linha.= str_repeat('0', 8);
            
            //Valor efetiva - Posição: 163 a 177 - Picture: 9(15)
            $linha.= str_repeat('0',15);
            
            //Finalidade Detalhe - Posição: 178 a 195 - Picture: X(18)
            $linha.= str_repeat(' ', 18);
            
            //Brancos - Posição: 196 a 197 - Picture: X(2)
            $linha.= str_repeat(' ', 2);
            
            //Numero de DOC/OP/CHEQUE - Posição: 198 a 203 - Picture: 9(6)
            $linha.= str_repeat('0', 6);
            
            //Numero de Inscriçao - Posição: 204 a 217 - Picture: 9(14)
            if ($fornecedor->getIdtpijurforn() == 'F'){
                if($boleto->getNrinscnabbop())
                    $linha.= str_pad(
                                substr(trim($boleto->getNrinscnabbop()),0,9) +
                                str_repeat('0', 4) +
                                substr(trim($boleto->getNrinscnabbop()),10,2)
                             , 15,'0',STR_PAD_LEFT);
                else
                    $linha.= str_pad(
                                substr(trim($fornecedor->getNrinsjurforn()),0,9) +
                                str_repeat('0', 4) +
                                substr(trim($fornecedor->getNrinsjurforn()),10,2)
                             , 15,'0',STR_PAD_LEFT);
            }else{
                if($boleto->getNrinscnabbop())
                    $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getNrinscnabbop())),0,15), 15,'0',STR_PAD_LEFT);
                else
                    $linha.= str_pad(substr(trim(Util::campoCnabValido($fornecedor->getNrinsjurforn())),0,15), 15,'0',STR_PAD_LEFT);
            }

            //NOME DO CAMPO:FINALIDADE DOC E STATUS FUNCIONÁRIO - POSIÇÃO:218 219 - PICTURE:X(02) NOTA 30
            $linha.= str_repeat(' ', 2);

            //NOME DO CAMPO:FINALIDADE TED
            //POSIÇÃO:220 224
            //PICTURE:X(05) NOTA 26
            $linha.= str_repeat(' ', 5);

            //NOME DO CAMPO:BRANCOS
            //POSIÇÃO:225 229
            //PICTURE:X(05)
            $linha.= str_repeat(' ', 5);

            //Aviso - Posição: 230 a 230 - Picture: X(1)
            $linha.= '0';
            
            //Ocorrências - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ', 10);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }

    }
    
    public function geraSegmentoBPIX($boleto, $idopecnabpag)
    {
        try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SICREDI;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= 'B';

            //TIPO CHAVE
            //Posição: 15 a 16 -
            //Picture: X(2)
            $linha.= $boleto->getIdchavepix();
            
            //Complemento do registro
            // Posicao 17 a 17
            //Picture X(1)
            $linha.= str_repeat(' ', 1);
            
            //Tipo Inscrição do Fornecedor
            // Posicao 18 a 18
            //Picture 9(1)
            $fornecedor = Fornecedor::findOneByNrorgAndCdforneced($this->nrorg, $boleto->getCdforneced());
            if ($fornecedor->getIdtpijurforn() == 'F')
                $linha.= '1';
            else
                $linha.= '2';

            //Nº DE INSCRIÇÃO FORNECEDOR
            // Posicao 19 a 32
            //Picture 9(14)
            if ($fornecedor->getIdtpijurforn() == 'F'){
                if($boleto->getNrinscnabbop())
                    $linha.= str_pad(
                                substr(trim($boleto->getNrinscnabbop()),0,9) +
                                str_repeat('0', 4) +
                                substr(trim($boleto->getNrinscnabbop()),10,2)
                             , 15,'0',STR_PAD_LEFT);
                else
                    $linha.= str_pad(
                                substr(trim($fornecedor->getNrinsjurforn()),0,9) +
                                str_repeat('0', 4) +
                                substr(trim($fornecedor->getNrinsjurforn()),10,2)
                             , 15,'0',STR_PAD_LEFT);
            }else{
                if($boleto->getNrinscnabbop())
                    $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getNrinscnabbop())),0,15), 15,'0',STR_PAD_LEFT);
                else
                    $linha.= str_pad(substr(trim(Util::campoCnabValido($fornecedor->getNrinsjurforn())),0,15), 15,'0',STR_PAD_LEFT);
            }
            
            // TXID - Opcional
            // Posicao 33 a 62
            // Picture X(30)
            $linha.= str_repeat(' ', 30);
            
            // INFORMAÇÕS ENTRE USUÁRIOS  - Opcional
            // Posicao 63 a 127
            // Picture X(65)
            $linha.= str_repeat(' ', 65);
            
            // CHAVE PIX  - obrigatorio
            // Posicao 128 a 227
            // Picture X(100)
            $linha.= str_pad($boleto->getChavepix(), 100);
            
            //Complemento do registro
            // Posicao 228 a 230
            //Picture X(3)
            $linha.= str_repeat(' ', 3);
            
            //Ocorrencias
            // Posicao 231 a 240
            //Picture X(10)
            $linha.= str_repeat(' ', 10);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }

    }

    public function geraSegmentoJPIX($boleto, $idopecnabpag)
    {
        try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SICREDI;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= 'J';

            //Tipo de movimento - Posição: 15 a 17 - Picture: 9(3)
            if ($idopecnabpag == 'E')
                $linha.= '999';
            else
                $linha.= '000';

            $cdbanco   = '000';                 //PARA PIX PREENCHER COM ZEROS
            $cdmoeda   = '0';                   //PARA PIX PREENCHER COM ZEROS
            $cddigito  = '0';                   //PARA PIX PREENCHER COM ZEROS
            $fatorvenc = '0000';                //PARA PIX PREENCHER COM ZEROS
            $valor     = str_repeat('0',10);    //PARA PIX PREENCHER COM ZEROS
            $cdcampo   = str_repeat('0',25);    //PARA PIX PREENCHER COM ZEROS
            
            //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
            $linha.= $cdbanco;

            //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
            $linha.= $cdmoeda;

            //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
            $linha.= $cddigito;

            //NOME DO CAMPO:VENCIMENTO CÓD. DE BARRAS – FATOR DE VENCIMENTO
            //POSIÇÃO:023 026
            //PICTURE:9(04) NOTA 18
            $linha.= $fatorvenc;

            //Valor (Cd. Barra) - Posição: 23 a 36 - Picture: 9(14)
            $linha.= $valor;

            //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
            $linha.= $cdcampo;

            //Nome do cedente - Posição: 62 a 91 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);

            //Data do vencimento - Posição: 92 a 99 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do Título - Posição: 100 a 114 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Valor do Desconto + Abatimento - Posição: 115 a 129 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0',15);

            //Valor do Acrescimo - Posição: 130 a 144 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0',15);

            //Data do pagamento - Posição: 145 a 152 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do pagto - Posição: 153 a 167 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Quantidade de moeda - Posição:  168 a 182 - Picture: 9(15)
            $linha.= str_repeat('0',15);

            //Cód. Refer. atribuído pelo sacado - Posição: 183 a 202 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                $boleto->getCdfilial(),
                $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);

            //Complemento - Posição: 203 a 215 - Picture: X(13)
            $linha.= str_repeat(' ',13);

            //Nosso Número - Posição: 216 a 230 - Picture: X(15)
            $linha.= str_repeat(' ',15);

            //Ocorrências - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }

    }

    public function geraSegmentoJ52PIX($boleto, $idopecnabpag)
    {
        try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_SICREDI;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= 'J';

            //Tipo de movimento - Posição: 15 a 17 - Picture: 9(3)
            if ($idopecnabpag == 'E')
                $linha.= '999';
            else
                $linha.= '000';

            //CAMPO: Identificação de Registro Opcional
            //POSIÇÃO: 018 a 019
            //PICTURE: 9(02)
            $linha.=  '52';
            
            //CAMPO: Tipo de Inscrição do Pagador
            //POSIÇÃO: 020 a 020
            //PICTURE: 9(1)
            $linha.= $this->tipoinscricao;

            //CAMPO: Número de Inscrição do Pagador
            //POSIÇÃO: 021 a 035
            //PICTURE: 9(15)
            $linha.= str_pad(trim($this->inscricaojuridica),15,'0',STR_PAD_LEFT);

            //CAMPO: Nome do Pagador
            //POSIÇÃO: 036 a 075
            //PICTURE: X(40)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,40),40);

            //CAMPO: Tipo de Inscrição do Beneficiário
            //POSIÇÃO: 076 a 076
            //PICTURE: 9(1)
            $fornecedor = Fornecedor::findOneByNrorgAndCdforneced($this->nrorg, $boleto->getCdforneced());
            if ($fornecedor->getIdtpijurforn() == 'F')
                $linha.= '1';
            else
                $linha.= '2';
            
            //CAMPO: Número de Inscrição do Beneficiário
            //POSIÇÃO: 077 a 091
            //PICTURE: 9(15)
            //(CGC/CPF do fornecedor) + (CGC <-> Filial) + (CGC/CPF <-> Controle).
            //(15 posições numericas)
            if ($fornecedor->getIdtpijurforn() == 'F'){
                if($boleto->getNrinscnabbop())
                    $linha.= str_pad(
                                substr(trim($boleto->getNrinscnabbop()),0,9) +
                                str_repeat('0', 4) +
                                substr(trim($boleto->getNrinscnabbop()),10,2)
                             , 15,'0',STR_PAD_LEFT);
                else
                    $linha.= str_pad(
                                substr(trim($fornecedor->getNrinsjurforn()),0,9) +
                                str_repeat('0', 4) +
                                substr(trim($fornecedor->getNrinsjurforn()),10,2)
                             , 15,'0',STR_PAD_LEFT);
            }else{
                if($boleto->getNrinscnabbop())
                    $linha.= str_pad(substr(trim(Util::campoCnabValido($boleto->getNrinscnabbop())),0,15), 15,'0',STR_PAD_LEFT);
                else
                    $linha.= str_pad(substr(trim(Util::campoCnabValido($fornecedor->getNrinsjurforn())),0,15), 15,'0',STR_PAD_LEFT);
            }
       
            //CAMPO: Nome do Beneficiário
            //POSIÇÃO: 092 a 131
            //PICTURE: X(40)
            $linha.= str_pad(Util::campoCnabValido(substr($fornecedor->getNmrazsocforn(),0,40)),40);

            //CAMPO: CHAVE DE PAGAMENTO
            //POSIÇÃO: 132 a 208
            //Picture: X(77)
            $linha.= str_pad($boleto->getCodeqrpix(),77);
            
            //CAMPO: TXID - Opcional
            //POSIÇÃO: 209 a 240
            //PICTURE: X(32)
            $linha.= str_repeat(' ', 32);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }

    }
    
    
    private function geraSegmentoB($boleto, $idopecnabpag){

        $this->qtdRegiLote++;
        $this->qtdRegiArq++;

        //Código do banco - Posição: 1 a 3 - Picture: 9(3)
        $linha = self::CODIGO_COMPENSACAO_SICREDI;

        //Código do lote de serviço - Posição: 4 a 7 - Picture: 9(4)
        $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

        //Tipo de registro = Regitro de detalhe do lote - Posição: 8 a 8 - Picture: 9(1)
        $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

        //Nº de sequencial registro no lote - Posição: 9 a 13 - Picture: 9(5)
        $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

        //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
        $linha.= 'B';

        //BRANCOS - Posição: 15 a 17 - Picture: 9(03)
        $linha.= str_repeat(' ', 3);

        //Tipo de incrição do fornecedor - Posição: 18 a 18 - Picture: 9(01)

        if($boleto->getIdtpicnabbop()) {
            if ($boleto->getIdtpicnabbop() == 'F')
                $tipoInsc = '1'; //CPF
            else {
                if ($boleto->getIdtpicnabbop() == 'J')
                    $tipoInsc =  '2'; //CNPJ
                else
                    $tipoInsc = '3'; //Outros
            }
            $linha.= $tipoInsc;
            
            // $linha.= str_pad(substr($boleto->getNrinscnabbop(), 0, 14), 14,'0',STR_PAD_LEFT);
        }
        else {
            if ($boleto->getForneced()->getIdtpijurforn() == 'F')
                $tipoInsc = '1'; //CPF
            else {
                if ($boleto->getForneced()->getIdtpijurforn() == 'J')
                    $tipoInsc = '2'; //CNPJ
                else
                 $tipoInsc = '3'; //Outros
            }
            $linha.= $tipoInsc;
            // $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(), 0, 14), 14,'0',STR_PAD_LEFT);
        }
        
        
        //Código do CPF/CNPJ - Posição: 19 a 32 - Picture: 9(14)
        if($tipoInsc == '1'){
            if($boleto->getNrinscnabbop()){
                $linha.= str_pad(substr($boleto->getNrinscnabbop(), 0, 14), 14,'0',STR_PAD_LEFT); 
            } else {
                $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(), 0, 14), 14,'0',STR_PAD_LEFT);
            }
        } else {
            $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(), 0, 14), 14,'0',STR_PAD_LEFT);
        }
    
        //Nome do logradouro do favorecido - Posição: 33 a 62 - Picture: X(30)
        $endforn = $boleto->getForneced()->getEndefornecobr();
        
        if(!$endforn)
            $endforn = $boleto->getForneced()->getEndeforneprinc();
            $linha.= str_pad(substr(Util::campoCnabValido($endforn->getDsendeforn()),0,30),30,' ',STR_PAD_RIGHT);

        //Número do imóvel do favorecido - Posição: 63 a 67 - Picture: 9(05)
        $linha.= str_repeat('0', 5);

        //Complemento do imóvel - Posição: 68 a 82 - Picture: X(15)
        $linha.= str_repeat(' ', 15);

        //Bairro do favorecido - Posição: 83 a 97 - Picture: 9(15)
        $linha.= str_pad(substr(Util::campoCnabValido($endforn->getNmbairforn()),0,15),15,' ',STR_PAD_RIGHT);

        //Município do favorecido - Posição: 98 a 117 - Picture: X(20)
        $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($endforn->getSgestado(), $endforn->getCdpais(), $endforn->getCdmunicipio());
        $linha.= str_pad(substr(Util::campoCnabValido($municipio->getNmmunicipio())
            ,0,20),20,' ',STR_PAD_RIGHT);

        //CEP - Posição: 118 a 122 - Picture: 9(05)
        if($endforn->getNrcepforn()){
            $linha.= str_pad(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),0,5),5,'0');
        }else{
            $linha.= str_repeat('0',5);
        }

        //Complemento do CEP - Posição: 123 a 125 - Picture: X(03)
        if(substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3)){
            $linha.= substr(Util::campoCnabValido(trim($endforn->getNrcepforn())),-3);
        }else{
            $linha.= str_repeat(' ',3);
        }

        //Estado - Posição: 126 a 127 - Picture: X(02)
        $linha.= str_pad(substr(Util::campoCnabValido($endforn->getSgestado()),0,2),2,' ',STR_PAD_RIGHT);

        //Data do vencimento (Nominal) - Posição: 128 a 135 - Picture: 9(08)
        $linha.= $boleto->getDtvenbolpag()->format('dmY');

        //Valor do documento (Nominal) - Posição: 136 a 150 - Picture: 9(13)V9(2)
        $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)),15,'0',STR_PAD_LEFT);

        //Valor do abatimento - Posição: 151 a 165 - Picture: 9(13)V9(2)
        $linha.= str_repeat('0', 15);

        //Valor do desconto - Posição: 166 a 180 - Picture: 9(13)V9(2)
        $linha.= str_repeat('0', 15);

        //Valor da mora - Posição: 181 a 195 - Picture: 9(13)V9(2)
        $linha.= str_repeat('0', 15);

        //Valor da multa - Posição: 196 a 210 - Picture: 9(13)V9(2)
        $linha.= str_repeat('0', 15);

        //Codigo do Favorecido - Posição: 211 a 225 - Picture: X(15)
        $linha.= str_repeat(' ', 15);

        //Aviso - Posição: 226 a 226 - Picture: X(1)
        $linha.= str_repeat('0', 1);
        
        //NOME DO CAMPO:Código UG Centralizadora- PICTURE:227 a 232 - POSIÇÃO:9(6)
        $linha.= str_repeat('0', 6);
        
        //NOME DO CAMPO:Identificação do Banco no SPB - PICTURE:233 a 240 - POSIÇÃO:9(8)
        $linha.= str_repeat('0', 8);
        
        return $linha."\r\n";
    }
}