<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\LeitorRetorno;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Model\Financeiro\Entities\Cnabrec;
use Teknisa\Model\Financeiro\Entities\Boletreceber;
use Teknisa\Model\Financeiro\Entities\Errcnabbolr;
use Teknisa\Model\Financeiro\Entities\Baixarecebida;
use Teknisa\Model\Financeiro\Entities\GeralRec;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Banco\RetornoCreator;
use Teknisa\FIN\Modules\ContaReceber\Factory\RetornoCobrancaFactory;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\CreditoBancarioService;
/**
 * Class RetornoCobrancaService
 * @package Teknisa\FIN\Modules\ContaReceber\Services
 */
class RetornoCobrancaService {
    CONST TIPO_CNAB_REMESSA = 0;
    CONST TIPO_CNAB_RETORNO = 1;

    CONST OPERACAO_INCLUSAO = 'I';
    CONST OPERACAO_EXCLUSAO = 'E';
    CONST OPERACAO_BLOQUEIO = 'B';

    CONST SITUAACAO_NAO_ENVIADO        = '00';
    CONST SITUACAO_ENVIADO_COM_ERRO    = '01';
    CONST SITUACAO_ENVIADO_SEM_RETORNO = '10';
    CONST SITUACAO_ENVIADO_RETORNO_OK  = '20';
    CONST SITUACAO_BAIXADO_VIA_CNAB    = '90';

    CONST CONTADOR_BAIXA = 'BAIXARECEBIDA';
    CONST CONTADOR_CNAB_RECEBIMENTO = 'CNABREC';

    CONST STATUS_PENDENTE_CNAB = 'C';
    CONST STATUS_PAGAMENTO_PLANEJADO = 'P';

    protected $entityManager;
    protected $environment;
    protected $retornoCobrancaFactory;
    protected $creditoBancarioService;
    protected $newCode;
    protected $baixaInserida;

    /**
     * @param EntityManager $entityManager
     * @param Environment $environment
     * @param RetornoCobrancaFactory $retornoCobrancaFactory
     * @param $newCode
     */
    public function __construct(EntityManager $entityManager, Environment $environment, RetornoCobrancaFactory $retornoCobrancaFactory, CreditoBancarioService $creditoBancarioService, NewCode $newCode)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->retornoCobrancaFactory = $retornoCobrancaFactory;
        $this->creditoBancarioService = $creditoBancarioService;
        $this->newCode = $newCode;
    }

    /**
     * Valida Layout Banco
     * @param $cdbanco
     * @param $cdempresa
     * @param $nrorg
     * @return string
     */
    public function validaLayoutBanco($cdbanco, $cdempresa, $nrorg){

        $emprbanc = $this->retornoCobrancaFactory->buscaEmprbanc($cdbanco, $cdempresa, $nrorg);
       // if (!$emprbanc){
      //      throw new \Exception('Cnab não parametrizada para o banco informado.');
      //  }
        return null;//$emprbanc->getIdpadrcnabrec();
    }
    /**
     * Carrega Retor
     * @param $row
     * @return array
     */
    public function carregaRetorno($row){
        $nrorg = isset($row['NRORG'])? $row['NRORG'] : $this->environment->getNrOrgTrab();
        // Variáveis de Inicialização
        $cdempresa   = $row['CDEMPRESA'];
        $cdbanccnab  = $row['CDBANCCNAB'];
        $cnabretorno = $row['CNABRETORNO'];
        // Decodifica Arquivo CNAB
        $cnabretorno = $this->decodeFile($cnabretorno);
        // Busca Linhas Arquivo CNAB
        $cnabretorno = $this->getStringInlines($cnabretorno);
        // Valida Layout CNAB Banco
        $layout = $this->validaLayoutBanco($cdbanccnab, $cdempresa, $nrorg);
        // Interpreta Arquivo CNAB
        $leitorCnab = RetornoCreator::getBancoCnab($cdempresa, $cdbanccnab, $cnabretorno, $nrorg, $layout);
        $leitorCnab->leArquivo();
        $leitorCnab->validaBoletos();
        // Busca Ocorrencia Arquivo CNAB
        $dataSet   = array();
        $dataSet[] = $leitorCnab->getRetornos();
        $dataSet[] = $leitorCnab->getErros();
        $dataSet[] = [$leitorCnab->getStatusArquivo()];
        return $dataSet;
    }

    /**
     * Decodificador Base 64
     * @param $encodedFile
     * @return string
     */
    public function decodeFile($encodedFile) {
        $file = base64_decode($encodedFile);
        return $file;
    }

    /**
     * Busca String em Linha
     * @param $file
     * @return array
     */
    public function getStringInlines($file) {
        $file = preg_split('/\R/', $file);
        return $file;
    }
    
    /**
     * Insere Retorno Cobrança
     * @param $boletos
     * @param $ocorrencias
     * @param $nomearquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function insereRetorno(
        $boletos, $ocorrencias, $nomearquivo, $cdempresa, $cdbanco,
        $nrorg = false, $cdfilial = false, $cdoperador = false, $importacao = false
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $this->insereRetornoCobranca(
                $boletos, $ocorrencias, $nomearquivo, $cdempresa, $cdbanco,
                $nrorg, $cdfilial, $cdoperador, $importacao
            );
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    /**
     * Insere Retorno Cobrança
     * @param $boletos
     * @param $ocorrencias
     * @param $nomearquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function insereRetornoCobranca(
        $boletos, $ocorrencias, $nomearquivo, $cdempresa, $cdbanco,
        $nrorg = false, $cdfilial = false, $cdoperador = false, $importacao = false
    ){
        try{
            // Variaveis de Inicialização
            $nrorg      = !$nrorg      ? $this->environment->getNrOrgTrab()  : $nrorg;
            $cdfilial   = !$cdfilial   ? $this->environment->getCdFilial()   : $cdfilial;
            $cdoperador = !$cdoperador ? $this->environment->getCdOperador() : $cdoperador;

            // Gera Novo Cógido
            $nrcnabrec = $this->newCode->getCodeByTableName(self::CONTADOR_CNAB_RECEBIMENTO.$cdempresa, $nrorg);
            // Insere Retorno Cobrança
            $cnabrec = $this->retornoCobrancaFactory->insereCnabrec(
                $nrorg, $cdempresa, $nrcnabrec, self::TIPO_CNAB_RETORNO, $cdoperador, $cdbanco, $nomearquivo
            );

            $this->entityManager->persist($cnabrec);
            $this->entityManager->flush();

            //Prepara Boleto p/ Inclusão
            $boletosBaixa = $this->preparaBoletoInclusao($boletos, $importacao);

            // Percorre Boletos p/ Inclusão
            foreach ($boletosBaixa as $data=>$boletosData) {
                //Loop nos tipos de baixa
                foreach ($boletosData as $tpbaixa=>$boletosTpBaixa) {
                    $prxbaixa = $this->newCode->getCodeByTableName(self::CONTADOR_BAIXA.$cdfilial, $nrorg);

                    //Variavel de controle para inserir uma baixa por Tipo de baixa
                    $this->baixaInserida = false;

                    foreach ($boletosTpBaixa as $key=>$boleto) {
                        $valortotal = 0;
                        if($boleto['IDSITUCNABR'] == self::SITUACAO_BAIXADO_VIA_CNAB) {
                            // Calcula Valor Total Pago
                            if (round($boleto['VRPAGO'], 2) != round($boleto['VRFINALBREC'], 2)) {
                                $valortotal += ($boleto['VRFINALBREC'] + $boleto['VRACRESCIMO'] - $boleto['VRDESCONTO']);
                            } else $valortotal += ($boleto['VRFINALBREC']);

                            // Valida Boleto Receber
                            $baixarecebida = $this->validaBaixaBoleto(
                                $data, $cdempresa, $cdfilial, $tpbaixa, $boleto['CDREDUCONT'], $cdoperador, $nrorg,
                                $prxbaixa, $valortotal, $nrcnabrec
                            );

                            /** @var GeralRec $geralrec */
                            $geralrec = $this->retornoCobrancaFactory->buscaGeralRec($nrorg);

                            // Insere Ajuste de Acréscimo
                            if ($geralrec->getCdajacrerec() && $boleto['VRACRESCIMO'] > 0){
                                $ajusteacres = $this->retornoCobrancaFactory->inserirAjusteBoleto($nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $geralrec->getCdajacrerec(), $boleto['VRACRESCIMO'], '3', '1');
                                $this->entityManager->persist($ajusteacres);
                            }

                            // Insere Ajuste de Desconto
                            if($geralrec->getCdajdescrec() && ($boleto['VRDESCONTO'] > 0)) {
                                $ajustedesc = $this->retornoCobrancaFactory->inserirAjusteBoleto($nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $geralrec->getCdajdescrec(), $boleto['VRDESCONTO'], '3', '1');
                                $this->entityManager->persist($ajustedesc);
                            }

                            $this->entityManager->flush();

                            // Atualiza Situação Boleto Receber
                            $this->atualizaSitBoletRec($boleto, $valortotal, $baixarecebida->getCdfilial(), $prxbaixa, $nrcnabrec, $nrorg);

                            // Verifica Existência Ocorrência(s)
                            if($ocorrencias)
                                $this->insereOcorrBoletRec($boleto, $ocorrencias, $cdbanco, $nrorg);
                                
                            // Atualiza Conta Receber
                            $this->atualizaContareceber($boleto, $nrorg);
                            
                        }else {
                            $this->associaBoletosNaoBaixadosCnab($boleto, $nrorg, $nrcnabrec);
                        }
                    }
                }
            }
            if(empty($boletosBaixa) && $importacao){
                var_dump('Não há boletos aptos a serem importados.');
                return false;
            } else {
                $this->entityManager->flush();
                return true;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function associaBoletosNaoBaixadosCnab($boleto, $nrorg, $nrcnabrec){
        $boletreceber = $this->retornoCobrancaFactory->buscaBoletreceber(
            $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $nrorg
        );
        if (($boleto['VRTXCART'] > 0) && ($boleto['IDSITUCNABR'] == '20')){
            $this->retornoCobrancaFactory->atualizaCarteiraBoletreceber($boletreceber, $boleto['DTOCORRENCIA'], $boleto['VRTAXACARTORIO']);
        }
        $boleto['IDCREDBANC'] = '11';
        if (substr($boleto['IDCREDBANC'], 1, 1) == '1'){
            if($boleto['CDREDUCONT']){
                $this->creditoBancarioService->updateValorUtilizadoContaCorrente($boleto['CDREDUCONT'], $boleto['VRFINALBREC']);
            }
        }
        $boleto['IDNOVACARTEIRA'] = '000';
        
        /*
         * Busca a posição 54 do segmento T
         * Foi necessario comentar a atualização do CDCARTCOBBOL e IDCARTCOBREC, passado por parametro,
         * porque não condiz com o numero da carteira mesmo e sim com o identificador
         */
        $cdcartcobbol = $boleto['CDCARTCOBBOL'];
        $idcartcobrec = $boleto['IDCARTCOBREC'];

        if ($cdcartcobbol){
            if (in_array(substr($cdcartcobbol, 0, 1), ['1', '2', '3', '4'])){
                $boleto['IDNOVACARTEIRA'] = $cdcartcobbol;
            } 
        }
        $dtiniprotrec = null;
        $idprotestrec = '';
        if ($boleto['IDCREDBANC'] == 'S'){
            if ($boleto['DTOCORRENCIA'] > 0){
                $dtiniprotrec = $boleto['DTOCORRENCIA'];
            }
        }else{
            $idprotestrec = 'N';
        }

        $boletreceber = $this->retornoCobrancaFactory->associaBoletoreceberCnab(
            $boletreceber, $cdcartcobbol, $idcartcobrec, $nrcnabrec, $boleto['IDSITUCNABR'],
            $dtiniprotrec, $idprotestrec, $nrorg
        );
        $this->entityManager->persist($boletreceber);
        $this->entityManager->flush();
    }

    /**
     * Prepara Boletos p/ Inclusão
     * @param $boletos
     * @return array
     * @throws \Exception
     */
    public function preparaBoletoInclusao($boletos, $importacao = false){
        $boletosporData = array();

        // Percorre Boletos a Receber
        foreach ($boletos as $boleto) {
            // Verifica Existência Situação Cnab
            if ($boleto['SITUACAOBOLETO']) {
                //Agrupa Boletos por Data de Pagamento
                $boletosporData[$boleto['DTPAGAMENTO']][] = $boleto;
            }
        }
        if((!$boletosporData) && $importacao === false){
            throw new \Exception('Não há boletos aptos a serem importados.');
        } else if((!$boletosporData) && $importacao){
            return [];
        }
        $boletosBaixa = array();

        // Percorre Boletos a Receber por Data de Pagamento
        foreach ($boletosporData as $key => $boletosData) {
            foreach ($boletosData as $boleto) {
                //Agrupa Boletos por Tipo de Baixa
                $boletosBaixa[$boleto['DTPAGAMENTO']][$boleto['CDTPBAIXAREC']][] = $boleto;
            }
        }

        return $boletosBaixa;
    }

    /**
     * @param $data
     * @param $cdempresa
     * @param $cdfilial
     * @param $tpbaixa
     * @param $cdreducont
     * @param $cdoperador
     * @param $nrorg
     * @param $nrbaixarec
     * @param $valortotal
     * @param $nrcnabrec
     * @return mixed|\Teknisa\Model\Financeiro\Entities\Baixarecebida
     * @throws \Exception
     */
    public function validaBaixaBoleto(
        $data, $cdempresa, $cdfilial, $tpbaixa, $cdreducont, $cdoperador, $nrorg, $nrbaixarec, $valortotal, $nrcnabrec
    ){
        if(!$this->baixaInserida){
            $dtpagamento = str_replace('/', '', $data);
            $dtpagamento = substr($dtpagamento, 0, 2).'/'.substr($dtpagamento, 2, 2).'/'.substr($dtpagamento, 4, 4);
            $dtpagamento = self::formataData($dtpagamento);

            /** @var Baixarecebida $baixarecebida */
            $baixarecebida = $this->retornoCobrancaFactory->insereBaixarecebida(
                $cdempresa, $cdfilial, $tpbaixa, $dtpagamento, $cdreducont, $cdoperador, $nrorg, $nrbaixarec,
                $valortotal, $nrcnabrec, self::STATUS_PENDENTE_CNAB
            );
            $this->baixaInserida = true;
        }else{
            /** @var Baixarecebida $baixarecebida */
            //Busca Baixa Recebida
            $baixarecebida = $this->retornoCobrancaFactory->buscaBaixarecebida($cdfilial, $nrbaixarec, $nrorg);

            //Atualiza Valor da Baixa Recebida
            $baixarecebida = $this->retornoCobrancaFactory->atualizaValorBaixarecebida($baixarecebida, $valortotal);
        }
        $this->entityManager->persist($baixarecebida);
        $this->entityManager->flush();

        return $baixarecebida;
    }

    /**
     * Atualiza Situação Boleto Receber
     * @param $boleto
     * @param $valortotal
     * @param $cdfilibairec
     * @param $nrbaixarec
     * @param $nrcnabrec
     * @param $nrorg
     */
    public function atualizaSitBoletRec($boleto, $valortotal, $cdfilibairec, $nrbaixarec, $nrcnabrec, $nrorg){
        /** @var Boletreceber $boletreceber */
        // Busca Boleto a Receber
        $boletreceber = $this->retornoCobrancaFactory->buscaBoletreceber(
            $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $nrorg
        );

        // Atualiza Boleto Receber
        $boletreceber = $this->retornoCobrancaFactory->atualizaBoletoreceber(
            $boletreceber, $valortotal, $cdfilibairec, $nrbaixarec, $nrcnabrec, $boleto['IDSITUCNABR'],
            $nrorg, self::STATUS_PAGAMENTO_PLANEJADO
        );

        $this->entityManager->persist($boletreceber);
        $this->entityManager->flush();
    }
    
    /**
     * Atualiza Conta Receber
     * @param $boleto
     * @param $nrorg
     */
    public function atualizaContareceber($boleto, $nrorg){
    	
    	// Busca Contas a Receber
    	/** @var Contareceber $contasreceber */
    	$contasreceber = $this->retornoCobrancaFactory->buscaContasreceber(
    		$nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC']
    	);
    	
    	/** @var Contareceber $contareceber */
    	foreach($contasreceber as $contareceber){
    		
    		$contareceber = $this->retornoCobrancaFactory->atualizaContareceber($contareceber, SELF::STATUS_PAGAMENTO_PLANEJADO);
    		
    		$this->entityManager->persist($contareceber);
    	}
    	
    	$this->entityManager->flush();
    }

    /**
     * Insere Ocorrencias Titulo
     * @param $boleto
     * @param $ocorrencias
     * @param $cdbanco
     * @param $nrorg
     */
    public function insereOcorrBoletRec($boleto, $ocorrencias, $cdbanco, $nrorg){
        // Busca Ocorrências Anteriores
        /** @var Errcnabbolr $erroscnab */
        $erroscnab = $this->retornoCobrancaFactory->buscaErrcnabbolrs(
            $boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $nrorg
        );

        // Percorre Ocorrências Anteriores
        foreach ($erroscnab as $errocnab)
            $this->entityManager->remove($errocnab);
        $this->entityManager->flush();
        // Percorre Novas Ocorrências
        foreach ($ocorrencias as $ocorrencia) {
            if($boleto['CDLINHA'] == $ocorrencia['CDLINHA']){
                // Insere Novas Ocorrências
                $erro = $this->retornoCobrancaFactory->insereErrcnabbolr(
                    $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $cdbanco, $ocorrencia['CDERRO'], $nrorg);
                $this->entityManager->persist($erro);
            }
        }
    }

    /**
     * Cancela Titulos do Retorno
     * @param $retorno
     * @param $boletos
     * @throws \Exception
     */
    public function cancelaTitulosRetornoCob($retorno, $boletos){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            foreach($boletos as $boleto)
                self::cancelaTituloRetornoCob($retorno, $boleto);

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
    /**
     * Cancela Titulo do Retorno
     * @param $retorno
     * @param $boleto
     * @throws \Exception
     */
    public function cancelaTituloRetornoCob($retorno, $boleto){
        try{
            // Busca CNAB Receber
            /** @var Cnabrec $cnabrec */
            $cnabrec = $this->retornoCobrancaFactory->buscaCnabrec(
                $retorno['NRORG'], $retorno['CDEMPRESA'], $retorno['NRCNABREC']
            );

            if($cnabrec) {
                // Busca Boleto Cobrança
                /** @var Boletreceber $boletreceber */
                $boletreceber = $this->retornoCobrancaFactory->buscaBoletreceberRetorno(
                    $boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $cnabrec->getNrcnabrec()
                );

                if ($boletreceber) {

                    // Busca Ocorrências do Boleto Cobrança
                    /** @var Errcnabbolr $errcnabbolrs */
                    $errcnabbolrs = $this->retornoCobrancaFactory->buscaErrcnabbolrs(
                        $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['NRORG']
                    );

                    // Exclui Ocorrências Existêntes do Boleto Cobrança
                    if ($errcnabbolrs)
                        foreach ($errcnabbolrs as $errcnabbolr)
                            $this->entityManager->remove($errcnabbolr);

                    // Atualiza Boleto Cobrança
                    $boletreceber->setNrcnabrecret('');
                    $boletreceber->setIdsitucnabr('00');
                    $this->entityManager->persist($boletreceber);

                    // Atualiza Retorno Cobrança
                    $cnabrec->setVrtotbolcnabrec($cnabrec->getVrtotbolcnabrec() - $boletreceber->getVrfinalbrec());
                    $cnabrec->setQtbolenvcnabrec($cnabrec->getQtbolenvcnabrec() - 1);
                    $this->entityManager->persist($cnabrec);
                }
            }

            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Formata Data
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public static function formataData(
        $data
    ){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

}