<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;

use DateTime;
use Exception;
use Doctrine\DBAL\Connection;
use Teknisa\FIN\Services\ContabilizacaoService;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Factories\Bco01400RecalculoLancamentoContaCorrenteFactory;

class Bco01400RecalculoLancamentoContaCorrenteService { 
  
  protected $factory;
  protected $lastSelectContopectblResult;
  protected $connection;
  protected $contabilizacaoService;
  
  const LANC_INVALIDO_CODE = 1;
  const LANC_FORA_DO_PERIODO_CODE = 2;
  
  public function __construct(Bco01400RecalculoLancamentoContaCorrenteFactory $factory, Connection $connection, ContabilizacaoService $contabilizacaoService) {
    $this->factory = $factory;
    $this->contabilizacaoService = $contabilizacaoService;
    $this->connection = $connection;
  }
  
  public function validateRow($row) {
    
    $this->lastSelectContopectblResult = $this->factory->selectContopectbl($row['CDEMPRESA']);
    
    if ($this->lastSelectContopectblResult) {
      
      $dtInicio = DateTime::createFromFormat('d/m/Y H:i:s', $this->lastSelectContopectblResult['DTINICCTBL']);
      
      $dtFinal = DateTime::createFromFormat('d/m/Y H:i:s', $this->lastSelectContopectblResult['DTFINACCTBL']);
      
      $dtMovicred = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTMOVICRED']);
      
      if (!(($dtInicio <= $dtMovicred) && ($dtFinal >= $dtMovicred))) {
        throw new Exception("Lançamento fora do período contábil", Bco01400RecalculoLancamentoContaCorrenteService::LANC_FORA_DO_PERIODO_CODE);
      }
      
    } else {
      throw new Exception("Lançamento inválido", Bco01400RecalculoLancamentoContaCorrenteService::LANC_INVALIDO_CODE);
    }
    
  }
  
  public function getLastSelectContopectblResult() {
    
    if (isset($this->lastSelectContopectblResult)) {
      return $this->lastSelectContopectblResult;
    } else {
      return false;
    }
    
  }
  
  public function getRowsToBeUnchecked($rows) {
    
    $results = array();
    $rowsToBeUnchecked = array();
    
    foreach($rows as $row) {
      
      if (!isset($result[$row['CDEMPRESA']])) {
        $results[$row['CDEMPRESA']] = $this->factory->selectContopectbl($row['CDEMPRESA']);
      }
      
      if ($results[$row['CDEMPRESA']]) {
        
        $dtInicio = DateTime::createFromFormat('d/m/Y H:i:s', $results[$row['CDEMPRESA']]['DTINICCTBL']);
      
        $dtFinal = DateTime::createFromFormat('d/m/Y H:i:s', $results[$row['CDEMPRESA']]['DTFINACCTBL']);
      
        $dtMovicred = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTMOVICRED']);
      
        if (!(($dtInicio <= $dtMovicred) && ($dtFinal >= $dtMovicred))) {
          $rowsToBeUnchecked[] = $row;
        }
      } else {
        $rowsToBeUnchecked[] = $row;
      }
      
    }
    
    return $rowsToBeUnchecked;
      
  }
  
  public function recontabilizarLancamentos($rows, $response) {
    
    $hadError = false;

    foreach($rows as $row) {
        
        $cdempresa = $row['CDEMPRESA'];
        $cdreducont = $row['CDREDUCONT'];
        $nrmovi = $row['NUMERO'];
        $idtipo = $row['TIPO'];
        $idctblbanc = $row['IDCONTABILIZA'];
        $this->connection->beginTransaction();
        try {
            $this->contabilizacaoService->contabilizaLancamentoControleBancario($cdempresa, $cdreducont, $nrmovi, $idtipo, $idctblbanc);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $hadError = true;
        }
        
    }
    
    if ($hadError) {
        $this->contabilizacaoService->saveContabilizacaoData($response);
        throw new \Exception("");
    }
    
  }
    
}